/***************************************************************************
*	 Copyright (C) 2004 by Christoph Thielecke														 *
*	 crissi99@gmx.de																											 *
*																																				 *
*	 Contains parts of code from klcc project															*
*	 (Peter Simonsson <psn@linux.se>.																			*
*																																				 *
*	 This program is free software; you can redistribute it and/or modify	*
*	 it under the terms of the GNU General Public License as published by	*
*	 the Free Software Foundation; either version 2 of the License, or		 *
*	 (at your option) any later version.																	 *
*																																				 *
*	 This program is distributed in the hope that it will be useful,			 *
*	 but WITHOUT ANY WARRANTY; without even the implied warranty of				*
*	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the				 *
*	 GNU General Public License for more details.													*
*																																				 *
*	 You should have received a copy of the GNU General Public License		 *
*	 along with this program; if not, write to the												 *
*	 Free Software Foundation, Inc.,																			 *
*	 59 Temple Place - Suite 330, Boston, MA	02111-1307, USA.						 *
***************************************************************************/ 
// BEGIN includes
#include <time.h>
#include <stdlib.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <qptrlist.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <qspinbox.h>
#include <qstringlist.h>
#include <qcombobox.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <qsocket.h>
#include <qstringlist.h>
#include <kstdaction.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpassdlg.h>
#include <knotifyclient.h>
#include <kstatusbar.h>
#include <kdebug.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <knotifydialog.h>
#include <kapplication.h>
#include <klineedit.h>
#include <kpassdlg.h>
#include <kurlrequester.h>
#include <kcombobox.h>
#include <kstddirs.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <klistview.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <knuminput.h>
#include "kvpnc.h"
#include "mainview.h"
#include "mainviewbase.h"
#include "preferencesdialog.h"
#include "helpdialog.h"
#include "advancedsettingsdialog.h"
#include "importprofiledialog.h"
#include "networkinterface.h"
#include "newprofiledialog.h"
#include "importp12certificatedialog.h"
#include "vpnaccountdata.h"
#include "profilemanager.h"
#include <ktempfile.h>
#include <iostream>

// END includes

KVpnc::KVpnc( KApplication *parent, QWidget*, const char *name ) : KMainWindow( 0, name ) {

	config = kapp->config();
	this->parent = parent;
	setCaption( i18n( "KVpnc" ) );

	disconnectedIcon = KGlobal::iconLoader() ->loadIcon( "kvpnc-disconnected", KIcon::NoGroup, 22 );
	connectedIcon = KGlobal::iconLoader() ->loadIcon( "kvpnc-connected", KIcon::NoGroup, 22 );

	setIcon( disconnectedIcon );

	KStandardDirs * dirs = KGlobal::dirs();
	logfileName = dirs->saveLocation( "data" );
	logfileName += "/kvpnc/kvpnc.log";
	logfile.setName( logfileName );
	s.setDevice( &logfile );

	this->initStatusbar();
	this->initAction();
	this->initDockWindow();
	this->initView();

	isConnected = false;
	mw->buttonConnect->setEnabled( false );
	mw->buttonDisconnect->setEnabled( false );
	minimizeAfterConnect = true;
	//showDebugConsole = false;
	useDefaultPaths = false;
	writeLogFile = true;
	useSilentQuit = false;
	holdGeneratedScripts = true; // TODO made gui option, currently allways true

	pathToVpnc = "";
	pathToRacoon = "";
	pathToSetkey = "";
	pathToFreeswan = "";
	lastProfile = "";
	Vpnc_pid_file = "";
	VpncConnectScript = "";
	VpncDisconnectScript = "";
	OldDefaultroute = "";

	connectCounter = 0;
	VpncDebugLevel = 0;
	RacoonDebugLevel = 0;
	KvpncDebugLevel = 0;
	connectSleepDelay = 15;
	connectSleepRetry = 0;
	tryConnectTimeout = 10;
	runningCheckCounter = 0;
	runningCheckTimeout = 60;
	currentProfile = 0;

	AccountList.setAutoDelete( TRUE ); // the list owns the objects
	this->loadOptions();
}

KVpnc::~KVpnc() {}

void KVpnc::initAction() {
	( void* ) KStdAction::quit( this, SLOT( quitCalled() ), actionCollection() );

	saveSessionAction = new KAction( i18n( "&Save profile..." ), "fileexport", Key_S, this, SLOT( saveSessionClicked() ), actionCollection(), "saveSession" );
	deleteSessionAction = new KAction( i18n( "&Delete profile..." ), "editdelete", Key_D, this, SLOT( deleteSessionClicked() ), actionCollection(), "deleteSession" );
	newSessionAction = new KAction( i18n( "&New profile..." ), "newfont", Key_N, this, SLOT( newSessionClicked() ), actionCollection(), "newSession" );
	//advancedSessionAction = new KAction( i18n( "&Advanced..." ), "wizard", Key_A, this, SLOT( advancedSettingsClicked() ), actionCollection(), "advancedSettingsSession" );
	importSessionAction = new KAction( i18n( "&Import Cisco pcf..." ), "kvpnc-pcfimport", Key_I, this, SLOT( importProfileClicked() ), actionCollection(), "importSession" );

	importCertAction = new KAction( i18n( "&Import P12 certificate..." ), "kvpnc-certimport", Key_C, this, SLOT( importCertClicked() ), actionCollection(), "importCert" );

	connectAction = new KAction( i18n( "&Connect" ), "up", CTRL + Key_Up, this, SLOT( connectClicked() ), actionCollection(), "serverConnect" );
	disconnectAction = new KAction( i18n( "&Disconnect" ), "down", CTRL + Key_Down, this, SLOT( disconnectClicked() ), actionCollection(), "serverDisconnect" );

	profileManagerAction = new KAction( i18n( "&Manage profiles" ), "view_detailed", CTRL + Key_M, this, SLOT( showProfileManagerClicked() ), actionCollection(), "profileManger" );

	KStdAction::preferences( this, SLOT( slotSettings() ), actionCollection(), "preferences" );

	KStdAction::helpContents( this, SLOT( helpClicked() ), actionCollection(), "help" );

	this->connectAction->setEnabled( true );
	this->disconnectAction->setEnabled( false );



	showToolbarAction = KStdAction::showToolbar( this, SLOT( slotShowToolbar() ), actionCollection() );
	showStatusbarAction = KStdAction::showStatusbar( this, SLOT( slotShowStatusbar() ), actionCollection() );
	preferencesAction = KStdAction::preferences( this, SLOT( slotSettings() ), actionCollection() );

	( void* ) KStdAction::keyBindings( this, SLOT( slotKeyBindings() ), actionCollection() );
	( void* ) KStdAction::configureToolbars( this, SLOT( slotConfigToolbars() ), actionCollection() );
	( void* ) KStdAction::configureNotifications( this, SLOT( slotConfigNotifications() ), actionCollection() );

	createGUI();

}

void KVpnc::initView() {

	mw = new MainView( this, "mainView" );
	setCentralWidget( mw );

	connect ( mw->buttonHelp, SIGNAL( clicked() ), this, SLOT( helpClicked() ) );
	connect ( mw->buttonConnect, SIGNAL( clicked() ), this, SLOT( connectClicked() ) );
	connect ( mw->buttonDisconnect, SIGNAL( clicked() ), this, SLOT( disconnectClicked() ) );

	connect ( mw->SessionCombo, SIGNAL ( activated ( const QString & ) ), this, SLOT ( sessionToggled( const QString& ) ) );

	//mw->DebugOutput->resize ( QSize( 0, 0 ) );
	//mw->DebugOutput->hide();
	//mw->resize( mw->minimumSizeHint() );


	//mw->CheckUserPass->setText(i18n("save user password"));
	//mw->CheckGroupPass->setText(i18n("save group password"));

	//	mw->SessionCombo->setEnabled( false );
	//mw->SaveSessionButton->setEnabled( false );
	//mw->SessionCombo->insertItem( "	 " ); // important: 3 spaces

	/*
	mw->CheckGroupPass->setEnabled ( FALSE );
	mw->CheckUserPass->setEnabled( FALSE );
	mw->GroupPasswordEdit->setEnabled( FALSE );
	mw->IDLineEdit->setEnabled( FALSE );
	mw->PasswordEdit->setEnabled( FALSE );
	mw->SessionCombo->setEnabled( FALSE );
	mw->UsernameLineEdit->setEnabled( FALSE );
	mw->gatewayLineEdit->setEnabled( FALSE );
	mw->SaveSessionButton->setEnabled( FALSE );
	mw->DeleteSessionButton->setEnabled( FALSE );
	*/
	mw->SessionCombo->setEnabled( FALSE );
	//	mw->AdvancedButton->setEnabled( false );
}

void KVpnc::initStatusbar() {
	statusBar() ->insertItem( i18n( "Ready." ), 1, 100 );
	statusBar() ->setItemAlignment( 1, AlignLeft | AlignVCenter );
	statusBar() ->insertItem( i18n( "Disconnected" ), 2, 0, true );
}

void KVpnc::initDockWindow() {
	this->kickerMenu = new KVpncKicker( this, "kickerIcon" );
	this->kickerMenu->menu() ->insertSeparator();
	this->connectAction->plug( this->kickerMenu->menu() );
	this->disconnectAction->plug( this->kickerMenu->menu() );

	this->kickerMenu->menu() ->insertSeparator();
	this->preferencesAction->plug( this->kickerMenu->menu() );
}

void KVpnc::saveOptions() {
	// delete configuration file
	KStandardDirs * dirs = KGlobal::dirs();
	QString filePath = dirs->findResource ( "config", "kvpncrc" );
	QFile::remove
		( filePath );
	this->config->setGroup( "General Options" );

	this->config->writeEntry( "Minimize after connect", minimizeAfterConnect );
	this->config->writeEntry( "Use silent quit", useSilentQuit );
	this->config->writeEntry( "Show debugconsole", showDebugConsole );
	this->config->writeEntry( "Vpnc debuglevel", VpncDebugLevel );
	this->config->writeEntry( "Pid file for vpnc ", Vpnc_pid_file );
	this->config->writeEntry( "Racoon debuglevel", RacoonDebugLevel );
	this->config->writeEntry( "Kvpnc debuglevel", KvpncDebugLevel );
	this->config->writeEntry( "Path to vpnc-bin", pathToVpnc );
	this->config->writeEntry( "Path to racoon", pathToRacoon );
	this->config->writeEntry( "Path to setkey", pathToSetkey );
	this->config->writeEntry( "Path to iptables", pathToIptables );
	this->config->writeEntry( "Path to openssl", pathToOpenssl );
	this->config->writeEntry( "Path to freeswan", pathToFreeswan );
	this->config->writeEntry( "Show Statusbar", showStatusbarAction->isChecked() );
	this->config->writeEntry( "Try connect timeout", tryConnectTimeout );
	this->config->writeEntry( "Last Profile", lastProfile );
	this->config->writeEntry( "Write log file", writeLogFile );
	this->config->writeEntry( "Hold generated scripts", holdGeneratedScripts );

	toolBar( "mainToolBar" ) ->saveSettings( this->config, "ToolBar Settings" );

	this->config->writeEntry( "Mainwindow width", QWidget::width() );
	this->config->writeEntry( "Mainwindow height", QWidget::height() );
	this->config->writeEntry( "Mainwindow position x", QWidget::x() );
	this->config->writeEntry( "Mainwindow position y", QWidget::y() );

	/* = user data = */
	/* old style: one profile: will be removed in future */
	/*
	this->config->setGroup( "Client Options" );
	this->config->writeEntry( "VPN Gateway", this->mw->gatewayLineEdit->text() );
	this->config->writeEntry( "VPN ID", this->mw->IDLineEdit->text() );
	this->config->writeEntry( "Username", this->mw->UsernameLineEdit->text() );
	if ( this->mw->CheckUserPass->isChecked() ) {
		this->config->writeEntry( "Password", this->mw->PasswordEdit->text() );
	}
	if ( this->mw->CheckGroupPass->isChecked() ) {
		this->config->writeEntry( "Group password", this->mw->GroupPasswordEdit->text() );
	}
	this->config->writeEntry( "Save user password", this->mw->CheckUserPass->isChecked() );
	this->config->writeEntry( "Save group password", this->mw->CheckGroupPass->isChecked() );
	*/
	/* new style: multiple profiles: not implemented */
	VpnAccountData *it;
	if ( !AccountList.isEmpty() ) {
		for ( it = AccountList.first(); it; it = AccountList.next() ) {
			if ( !it->getName().isEmpty() ) {
				QString name = it->getName();
				QString ProfileName = "Profile_";
				ProfileName += name;
				this->config->setGroup( ProfileName );

				if ( it->getConnectionType() == VpnAccountData::cisco )
					this->config->writeEntry( "Connection type", "cisco" );
				else if ( it->getConnectionType() == VpnAccountData::racoon )
					this->config->writeEntry( "Connection type", "racoon" );
				else if ( it->getConnectionType() == VpnAccountData::freeswan )
					this->config->writeEntry( "Connection type", "freeswan" );
				else
					this->config->writeEntry( "Connection type", "other" );
				this->config->writeEntry( "VPN Gateway", it->getGateway() );
				this->config->writeEntry( "VPN ID", it->getID() );
				this->config->writeEntry( "Username", it->getUserName() );
				if ( it->getSaveUserPassword() ) {
					this->config->writeEntry( "User password", it->getUserPassword() );
				}
				if ( it->getSaveGroupPassword() ) {
					this->config->writeEntry( "Group password", it->getGroupPassword() );
				}
				this->config->writeEntry( "Save user password", it->getSaveUserPassword() );
				this->config->writeEntry( "Save group password", it->getSaveGroupPassword() );

				this->config->writeEntry( "Use advanced settings", it->getUseAdvancedSettings() );
				this->config->writeEntry( "Application version", it->getApplicationVersion() );
				this->config->writeEntry( "Local port", it->getLocalPort() );
				this->config->writeEntry( "NT domain name", it->getNtDomainName() );
				this->config->writeEntry( "Perfect forward secrety", it->getPerfectForwardSecrety() );
				this->config->writeEntry( "IKE group", it->getIkeGroup() );
				this->config->writeEntry( "Use application version", it->getUseApplicationVersion() );
				this->config->writeEntry( "Use global IPSec secret", it->getUseGlobalIpsecSecret() );
				this->config->writeEntry( "Use IKE group", it->getUseIkeGroup() );
				this->config->writeEntry( "Use local port", it->getUseLocalPort() );
				this->config->writeEntry( "Use NT domain name", it->getUseNtDomainName() );
				this->config->writeEntry( "Use single DES", it->getUseSingleDes() );
				this->config->writeEntry( "Use perfect Forward Secrety", it->getUsePerfectForwardSecrety() );
				QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
				QString RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 );
				this->config->writeEntry( "Remote net address", RemoteNetAddr );
				this->config->writeEntry( "Remote net mask", RemoteNetMask );
				this->config->writeEntry( "Pre shared key", it->getPreSharedKey() );
				this->config->writeEntry( "x509 certificate", it->getX509Certificate() );
				this->config->writeEntry( "Network device", it->getNetworkDevice() );
				this->config->writeEntry( "Cert path", it->getCertPath() );
				if ( it->getAuthType() == VpnAccountData::cert )
					this->config->writeEntry( "Auth type", "cert" );
				else
					this->config->writeEntry( "Auth type", "psk" );
				this->config->writeEntry( "do ping ip", it->getDoPingIP() );
				this->config->writeEntry( "Ping host IP", it->getPingHostIP() );
				this->config->writeEntry( "PrivateKey password", it->getPrivateKeyPass() );
				this->config->writeEntry( "Private key", it->getPrivateKey() );
				this->config->writeEntry( "Use special remote ID", it->getUseSpecialRemoteID() );
				this->config->writeEntry( "Special remote ID", it->getSpecialRemoteID() );

				slotStatusMsg ( i18n( "Profile \"%1\" saved." ).arg( ProfileName ), ID_FLASH_MSG );
				appendLogEntry ( i18n( "Profile \"%1\" saved." ).arg( ProfileName ) );
			}
		}
	}

	this->config->sync();
}

void KVpnc::loadOptions() {
	toolBar( "mainToolBar" ) ->applySettings( this->config, "ToolBar Settings" );
	showToolbarAction->setChecked( !toolBar( "mainToolBar" ) ->isHidden() );

	this->config->setGroup( "General Options" );
	minimizeAfterConnect = this->config->readBoolEntry( "Minimize after connect", true );
	useSilentQuit = this->config->readBoolEntry( "Use silent quit", false );
	showDebugConsole = this->config->readBoolEntry( "Show debugconsole", false );
	VpncDebugLevel = this->config->readNumEntry( "Vpnc debuglevel", 0 );
	Vpnc_pid_file = this->config->readEntry( "Pid file for vpnc ", "/var/run/vpnc/pid" );
	RacoonDebugLevel = this->config->readNumEntry( "Racoon debuglevel", 0 );
	KvpncDebugLevel = this->config->readNumEntry( "Kvpnc debuglevel", 0 );
	pathToVpnc = this->config->readEntry( "Path to vpnc-bin", "/usr/sbin/vpnc" );
	pathToSetkey = this->config->readEntry( "Path to setkey", "/usr/sbin/setkey" );
	pathToRacoon = this->config->readEntry( "Path to racoon", "/usr/sbin/racoon" );
	pathToIptables = this->config->readEntry( "Path to iptables", "/usr/bin/iptables" );
	pathToOpenssl = this->config->readEntry( "Path to openssl", "/usr/bin/openssl" );
	pathToFreeswan = this->config->readEntry( "Path to freeswan", "/usr/sbin/ipsec" );
	tryConnectTimeout = QString( this->config->readEntry( "Try connect timeout", "10" ) ).toInt();
	lastProfile = this->config->readEntry( "Last Profile", "" );
	writeLogFile = this->config->readBoolEntry( "Write log file", true );
	holdGeneratedScripts = this->config->readBoolEntry( "Hold generated scripts", true );
	bool bViewStatusbar = this->config->readBoolEntry( "Show Statusbar", true );

	showStatusbarAction->setChecked( bViewStatusbar );
	slotShowStatusbar();

	if ( !showDebugConsole )
		mw->DebugOutput->hide();
	else
		mw->DebugOutput->show();

	WindowSize = QSize( this->config->readNumEntry( "Mainwindow width", 50 ), this->config->readNumEntry( "Mainwindow height", 50 ) );
	WindowPos = QPoint ( this->config->readNumEntry( "Mainwindow position x", 10 ), this->config->readNumEntry( "Mainwindow position y", 10 ) );

	if ( WindowSize.width() < 50 || WindowSize.height() < 50 ) {
		WindowSize = QSize( minimumWidth(), minimumHeight() );
	}

	if ( WindowPos.x() < 0 || WindowPos.y() < 0 )
		WindowPos = QPoint ( 0, 0 );

	resize( WindowSize );
	move ( WindowPos );

	if ( writeLogFile )
		if ( !logfile.isOpen() ) {
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc.log";
			//KMessageBox::information( this, logfileName, "info" );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else {
				QTextStream s( &logfile );
				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}

	if ( KvpncDebugLevel > 0 )
		appendLogEntry( "debug: " + i18n( "Global configuration loaded." ) );

	//Failsafe
	if ( this->isHidden() && this->kickerMenu->isHidden() ) {
		this->show();
	}

	/* = user data = */
	/* old style: one profile: will be removed in future */
	/*
	this->config->setGroup( "Client Options" );
	this->mw->gatewayLineEdit->setText( this->config->readEntry( "VPN Gateway", "" ) );
	this->mw->IDLineEdit->setText( this->config->readEntry( "VPN ID", "" ) );
	this->mw->GroupPasswordEdit->setText( this->config->readEntry( "Group password", "" ) );
	this->mw->UsernameLineEdit->setText( this->config->readEntry( "Username", "" ) );
	this->mw->PasswordEdit->setText( this->config->readEntry( "Password", "" ) );

	this->mw->CheckUserPass->setChecked( this->config->readBoolEntry( "Save user password", false ) );
	this->mw->CheckGroupPass->setChecked( this->config->readBoolEntry( "Save group password", false ) );
	*/

	/* profiles are stored in a group which is named to the following scheme:
		Profile_XXXXX
		XXXXX is the account name (important: unique!)
	*/
	QStringList configGroups = this->config->groupList();
	bool oldConfigurationFound = false;

	bool sessionFound = false;

	for ( QStringList::Iterator it = configGroups.begin(); it != configGroups.end(); ++it ) {
		QString name = *it;
		if ( name.left( 7 ) == "Profile" && name.length() > 8 ) {
			// Profile found

			sessionFound = true;

			if ( KvpncDebugLevel > 0 ) {
				QString tmp = i18n( "debug: " + i18n( "Profile found: " ) );
				tmp += name.right( ( name.length() ) - 8 );
				appendLogEntry ( tmp );
			}
			this->config->setGroup( name );
			VpnAccountData::ConnectionType ConnType = VpnAccountData::ConnectionType( VpnAccountData::cisco );
			QString type_string = this->config->readEntry( "Connection type", "cisco" );
			if ( type_string == "cisco" )
				ConnType = VpnAccountData::cisco;
			else if ( type_string == "racoon" )
				ConnType = VpnAccountData::racoon;
			else if ( type_string == "freeswan" )
				ConnType = VpnAccountData::freeswan;
			else
				ConnType = VpnAccountData::other;
			QString Name = name.right( name.length() - 8 );
			QString Gateway = this->config->readEntry( "VPN Gateway", "" );
			QString ID = this->config->readEntry( "VPN ID", "" ) ;
			QString GroupPassword = this->config->readEntry( "Group password", "" );
			QString Username = this->config->readEntry( "Username", "" );
			QString UserPassword = this->config->readEntry( "User password", "" );
			bool saveUserPassword = this->config->readBoolEntry( "Save user password", false );
			bool saveGroupPassword = this->config->readBoolEntry( "Save group password", false );


			bool useAdvancedSettings = this->config->readBoolEntry( "Use advanced settings", false );
			QString ApplicationVersion = this->config->readEntry( "Application version", "Cisco Systems VPN Client 0.2-rm+zomb.1:Linux" );
			int Port = this->config->readNumEntry( "Local port", 500 );
			QString NtDomainName = this->config->readEntry( "NT domain name", "" );
			QString PerfectForwardSecrecy = this->config->readEntry( "Perfect forward secrety", "server" );
			QString IkeGroup = this->config->readEntry( "IKE group", "dh1" );
			bool useApplicationVersion = this->config->readBoolEntry( "Use application version", false );
			bool useGlobalIpsecSecret = this->config->readBoolEntry( "Use global IPSec secret", false );
			bool useIkeGroup = this->config->readBoolEntry( "Use IKE group", false );
			bool useLocalPort = this->config->readBoolEntry( "Use local port", false );
			bool useNtDomainName = this->config->readBoolEntry( "Use NT domain name", false );
			bool useSingleDes = this->config->readBoolEntry( "Use single DES", false );
			bool usePerfectSecrecy = this->config->readBoolEntry( "Use perfect Forward Secrety", false );
			QString RemoteNetAddr = this->config->readEntry( "Remote net address", "" );
			int RemoteNetMask = this->config->readNumEntry( "Remote net mask", 24 );
			QString PreSharedKey = this->config->readEntry( "Pre shared key", "" );
			QString X509Certificate = this->config->readEntry( "x509 certificate", "" );
			QString NetworkDevice = this->config->readEntry( "Network device", "default" );
			QString CertPath = this->config->readEntry( "Cert path", "/etc/racoon/certs" );
			QString PrivateKeyPass = this->config->readEntry( "PrivateKey password", "" );
			QString PrivateKey = this->config->readEntry( "Private key", "" );
			VpnAccountData::AuthenticationType authtype;
			if ( this->config->readEntry( "Auth type", "cert" ) == "cert" )
				authtype = VpnAccountData::cert;
			else
				authtype = VpnAccountData::psk;

			bool doPingIP = this->config->readBoolEntry( "do ping ip", false );
			QString pingHostIP = this->config->readEntry( "Ping host IP", "" );
			bool useSpecialRemoteID = this->config->readBoolEntry( "Use special remote ID", false );
			QString SpecialRemoteID = this->config->readEntry( "Special remote ID", "" );

			VpnAccountData *data = new VpnAccountData( ConnType, Name, Gateway, ID, GroupPassword, Username,
			                       UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings, usePerfectSecrecy );

			data->setRemoteNet( RemoteNetAddr + "/" + QString().setNum( RemoteNetMask ) );
			data->setPreSharedKey( PreSharedKey );
			data->setX509Certificate( X509Certificate );
			data->setCertPath( CertPath );
			data->setAuthType( authtype );
			data->setNetworkDevice( NetworkDevice );
			data->setDoPingIP( doPingIP );
			data->setPingHostIP( pingHostIP );
			data->setPrivateKey( PrivateKey );
			data->setPrivateKeyPass( PrivateKeyPass );
			data->setUseSpecialRemoteID( useSpecialRemoteID );
			data->setSpecialRemoteID( SpecialRemoteID );

			AccountList.append( data );



			//ConnType, Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings ) );

			if ( !Name.isEmpty() )
				mw->SessionCombo->insertItem( Name );
		}

		if ( sessionFound ) {
			mw->SessionCombo->setEnabled( TRUE );
			mw->buttonConnect->setEnabled( TRUE );
		}

		if ( name == "Client Options" ) {
			// old style configuration found
			this->config->setGroup( "Client Options" );
			QString Name = "Default";
			VpnAccountData::ConnectionType ConnType = VpnAccountData::cisco;
			QString Gateway = this->config->readEntry( "VPN Gateway", "" );
			QString ID = this->config->readEntry( "VPN ID", "" );
			QString GroupPassword = this->config->readEntry( "Group password", "" );
			QString Username = this->config->readEntry( "Username", "" );
			QString UserPassword = this->config->readEntry( "Password", "" );
			bool saveUserPassword = this->config->readBoolEntry( "Save user password", false );
			bool saveGroupPassword = this->config->readBoolEntry( "Save group password", false );


			AccountList.append( new VpnAccountData( ConnType, Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, QString( "" ), 500, QString( "" ), QString( "server" ), QString( "dh1" ), false, false, false, false, false, false, false, false ) );

			mw->SessionCombo->insertItem( Name );
			mw->SessionCombo->setCurrentItem( Name );

			lastProfile = "Default";

			sessionToggled( "" );

			// now mark for deleting old configuration
			oldConfigurationFound = true;
			if ( KvpncDebugLevel > 0 )
				appendLogEntry ( "debug: " + i18n( "Old configuration found, converted." ) );

			if ( KvpncDebugLevel > 0 )
				appendLogEntry( "debug: " + i18n( "Configuration for profile \"%1\" loaded." ).arg( Name ) );
		}


	}
	if ( oldConfigurationFound ) {
		// delete configuration file
		KStandardDirs * dirs = KGlobal::dirs();
		QString filePath = dirs->findResource ( "config", "kvpncrc" );
		QFile::remove
			( filePath );
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "Old configuration deleted." ) );
	}

	if ( !lastProfile.isEmpty() ) {
		mw->SessionCombo->setCurrentText( lastProfile );
		sessionToggled( "" );
	} else {
		if ( !AccountList.isEmpty() ) {
			mw->SessionCombo->setCurrentItem( 0 );
			currentProfile = AccountList.first();
		}
	}

	//mw->minimumSizeHint();

}

bool KVpnc::queryExit() {
	lastProfile = mw->SessionCombo->currentText();
	this->saveOptions();
	return true;
}

void KVpnc::slotSettings() {
	this->slotStatusMsg( i18n( "Setup KVpnc..." ), ID_FLASH_MSG );
	this->slotStatusMsg( i18n( "Ready." ), ID_STATUS_MSG );

	PreferencesDialog prefDlg( AccountList, mw->DebugOutput, KvpncDebugLevel, currentProfile, this, "PreferencesDlg" );
	prefDlg.VpncPathInput->setURL( pathToVpnc );
	prefDlg.RacoonPathInput->setURL( pathToRacoon );
	prefDlg.SetkeyPathInput->setURL( pathToSetkey );
	prefDlg.IptablesPathInput->setURL( pathToIptables );
	prefDlg.FreeswanPathInput->setURL( pathToFreeswan );
	prefDlg.OpenSSLPathInput->setURL( pathToOpenssl );
	prefDlg.checkMinimizeAfterConnectCheck->setChecked( minimizeAfterConnect );
	prefDlg.showDebugCheck->setChecked( showDebugConsole );
	prefDlg.WriteLogCheck->setChecked( writeLogFile );
	prefDlg.spinVpncDebugLevel->setValue ( VpncDebugLevel );
	prefDlg.spinRacoonDebugLevel->setValue ( RacoonDebugLevel );
	prefDlg.spinKvpncDebugLevel->setValue ( KvpncDebugLevel );
	prefDlg.spinTryConnectTimeout->setValue( tryConnectTimeout );
	prefDlg.SilentQuitcheckBox->setChecked( useSilentQuit );
	prefDlg.buttonHelp->hide();

	prefDlg.exec();

	pathToVpnc = prefDlg.VpncPathInput->url();
	pathToSetkey = prefDlg.SetkeyPathInput->url();
	pathToRacoon = prefDlg.RacoonPathInput->url();
	pathToIptables = prefDlg.IptablesPathInput->url();
	pathToOpenssl = prefDlg.OpenSSLPathInput->url();
	pathToFreeswan = prefDlg.FreeswanPathInput->url();
	minimizeAfterConnect = prefDlg.checkMinimizeAfterConnectCheck->isChecked();
	showDebugConsole = prefDlg.showDebugCheck->isChecked();
	useSilentQuit = prefDlg.SilentQuitcheckBox->isChecked();
	if ( showDebugConsole ) {
		mw->DebugOutput->show();
		//resize( minimumSizeHint() );
	} else {
		mw->DebugOutput->hide();
		/*
		mw->groupBox1->minimumSizeHint();
		mw->DebugOutput->resize( QSize( 0, 0 ) );
		resize( minimumSizeHint() );
		*/
	}
	VpncDebugLevel = prefDlg.spinVpncDebugLevel->value();
	KvpncDebugLevel = prefDlg.spinKvpncDebugLevel->value();
	tryConnectTimeout = prefDlg.spinTryConnectTimeout->value();

	if ( writeLogFile ) {
		if ( !logfile.isOpen() ) {
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc.log";
			//KMessageBox::information( this, "info", logfileName );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else {
				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}
	} else {
		if ( logfile.isOpen() ) {
			QTextStream s( &logfile );
			s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			logfile.close();
		}
	}
	saveOptions();
}

void KVpnc::slotShowToolbar() {
	if ( !showToolbarAction->isChecked() ) {
		toolBar( "mainToolBar" ) ->hide();
	} else {
		toolBar( "mainToolBar" ) ->show();
	}
}

void KVpnc::slotShowStatusbar() {

	if ( !showStatusbarAction->isChecked() ) {
		statusBar() ->hide();
	} else {
		statusBar() ->show();
	}

}

void KVpnc::slotKeyBindings() {
	//	KKeyDialog::configureKeys( actionCollection(), xmlFile() );
}

void KVpnc::slotConfigToolbars() {
	KEditToolbar dlg( actionCollection() );

	if ( dlg.exec() ) {
		createGUI();
	}
}

void KVpnc::slotConfigNotifications() {
	// 	KNotifyDialog::configure( this, "notifyDialog" );
}

void KVpnc::slotStatusMsg( const QString &text, int id ) {
	if ( id != ID_FLASH_MSG ) {
		statusBar() ->clear();
		statusBar() ->changeItem( text, id );
	} else {
		statusBar() ->message( text, 1000 );

	}

}

void KVpnc:: quitCalled() {
	bool reallyQuit = false;

	WindowSize = QSize( QWidget::width(), QWidget::height() );
	WindowPos = QPoint( this->x(), this->y() );

	if ( this->isConnected ) {
		int res;
		if ( !useSilentQuit ) {
			res = KMessageBox::questionYesNo ( this, i18n( "You are still conneced. Really quit?" ), i18n( "Quit?" ) );
			//KMessageBox::error( this,QString().setNum(res) );
			if ( res != 4 ) {
				disconnectClicked();
				reallyQuit = true;
				if ( writeLogFile ) {
					if ( logfile.isOpen() ) {
						QTextStream s( &logfile );
						s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
						logfile.flush();
						logfile.close();
					}
				}
				kapp->quit();
			}
		} else {
			kapp->quit();
		}
	} else {
		reallyQuit = true;
		if ( reallyQuit ) {
			if ( writeLogFile ) {
				if ( logfile.isOpen() ) {
					QTextStream s( &logfile );
					s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
					logfile.flush();
					logfile.close();
				}
			}
			kapp->quit();
		}
	}
}

void KVpnc::closeEvent( QCloseEvent* e ) {
	bool reallyQuit = false;
	if ( this->isConnected ) {
		int res;
		if ( !useSilentQuit ) {
			res = KMessageBox::questionYesNo ( this, i18n( "You are still conneced. Really quit?" ), i18n( "Quit?" ) );
			//KMessageBox::error( this,QString().setNum(res) );
			if ( res != 4 ) {
				reallyQuit = true;
				disconnectClicked();

				if ( reallyQuit ) {

					saveOptions();

					if ( writeLogFile ) {
						if ( logfile.isOpen() ) {
							QTextStream s( &logfile );
							s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
							logfile.close();
						}
					}
					KMainWindow::closeEvent( e );
				} else {
					e = 0L;
				}

			} else {
				e = 0L;
			}
		}
	} else {
		// not connected
		reallyQuit = true;
	}

	if ( reallyQuit ) {
		saveOptions();
		if ( writeLogFile ) {
			if ( logfile.isOpen() ) {
				s << i18n( "Log session ended at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
				logfile.flush();
				logfile.close();
			}
		}
		//KMainWindow::closeEvent( e );
		kapp->quit();
	}

}

void KVpnc::connectClicked() {
	bool ok = true; // if ok we will run process

	// type selection
	/* ============ vpnc ================ */
	if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::cisco ) {

		// looking for vpnc
		if ( useDefaultPaths || pathToVpnc.isEmpty() ) {
			QFile vpnc( "/usr/sbin/vpnc" );
			if ( vpnc.exists() )
				pathToVpnc = "/usr/sbin/vpnc";
			else {
				vpnc.setName( "/usr/local/sbin/vpnc" );
				if ( vpnc.exists() )
					pathToVpnc = "/usr/local/sbin/vpnc";
				else {
					vpnc.setName( "/usr/bin/vpnc" );
					if ( vpnc.exists() )
						pathToVpnc = "/usr/bin/vpnc";
					else {
						KMessageBox::error( this, i18n( "Unable to find vpnc!" ) );
						ok = false;
					}
				}
			}
		}

		if ( VpncDebugLevel > 0 )
			appendLogEntry( i18n( "vpnc: %1" ) .arg( pathToVpnc ) );

		// here we go
		if ( ok ) {

			tmpPath = locateLocal ( "data", "kvpnc/" );

			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface;
			if ( currentProfile->getNetworkDevice() == "default" ) {
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			} else {
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() ) {
				if ( KvpncDebugLevel > 0 )
					appendLogEntry( "debug: " + i18n( "No default interface found, using \"lo\"." ) );
				tmpInterface = "lo"; // no default interface found :(
			}

			if ( LocalIP.isEmpty() ) {
				if ( KvpncDebugLevel > 0 )
					appendLogEntry( "debug: " + i18n( "No IP for default interface found, using \"127.0.0.1\"." ) );
				LocalIP = "127.0.0.1"; // no IP for default interface found :(
			}

			//			QString tunneldevice = "tun0";
			int mtusize = 1412;
			OldDefaultroute = tmpPath + "default-route." + currentProfile->getID(); // default-route.<id>
			VpncGateway = tmpPath + "gateway." + currentProfile->getID(); // gateway.<id>
			VpncConnectScript = tmpPath + "vpnc-connect." + currentProfile->getID() + ".sh"; // vpnc-connect.<id>.sh
			Vpnc_pid_file = tmpPath + "vpnc-pid." + currentProfile->getID() + ".pid"; // vpnc-pid.<id>.pid

			// write vpnc connect script
			VpncScript = tmpPath + "vpnc-script." + currentProfile->getID(); // vpnc-script.<id>
			mw->DebugOutput->append( "VpncScript: " + VpncScript + "\n" );
			QFile file ( VpncScript );
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) ) {
				// 				stream << "#!/bin/bash" << "\n";
				// 				stream << "# generated by kvpnc. Do not edit it." << "\n";
				// 				stream << "# profile: " + currentProfile->getID() << "\n";
				// 				stream << "" << "\n";
				stream << "fix_ip_get_output () {\n";
				stream << "sed 's/cache//;s/metric[0-9]\\+ [0-9]\\+//g' | xargs echo\n";
				stream << "}\n";
				stream << "\n";
				stream << "ifconfig $TUNDEV inet $INTERNAL_IP4_ADDRESS pointopoint $INTERNAL_IP4_ADDRESS netmask 255.255.255.255 mtu " << mtusize << " up\n";
				stream << "ip route add $(ip route get $VPNGATEWAY | fix_ip_get_output)\n";
				stream << "ip route | grep '^default' | fix_ip_get_output > \"" + OldDefaultroute + "\"\n";
				stream << "echo \"$VPNGATEWAY\" > \"" + VpncGateway + "\"\n";
				stream << "ip route del default\n";
				stream << "ip route add default dev $TUNDEV\n";
				stream << "ip route flush cache\n";
			}
			file.close();

			ConnectProcess = new QProcess ( this );
			ConnectProcess->addArgument( "/bin/chmod" );
			ConnectProcess->addArgument( "+x" );
			ConnectProcess->addArgument( VpncScript );

			if ( ConnectProcess->start() ) {

				sleep ( 1 );


				ConnectProcess = new QProcess ( this );

				ConnectProcess->addArgument( pathToVpnc );
				//	ConnectProcess->addArgument("--no-detach");

				if ( VpncDebugLevel > 0 ) {

					ConnectProcess->addArgument( "--debug" );
					ConnectProcess->addArgument( QString().setNum( VpncDebugLevel ) );
				}
				ConnectProcess->addArgument( "--pid-file" );
				ConnectProcess->addArgument( Vpnc_pid_file );
				// 			ConnectProcess->addArgument( "--gateway" );
				// 			ConnectProcess->addArgument( currentProfile->getGateway() );
				// 			ConnectProcess->addArgument( "--id" );
				// 			ConnectProcess->addArgument( currentProfile->getID() );
				// 			ConnectProcess->addArgument( "--username" );
				// 			ConnectProcess->addArgument( currentProfile->getUserName() );
				ConnectProcess->addArgument( "--script" );
				ConnectProcess->addArgument( VpncScript );

				if ( currentProfile->getUseAdvancedSettings() ) {

					if ( currentProfile->getUseApplicationVersion() ) {
						ConnectProcess->addArgument( "--application-version" );
						ConnectProcess->addArgument( currentProfile->getApplicationVersion() );
						appendLogEntry ( i18n( "Using userdefinied application version \"%1\"." ).arg( currentProfile->getApplicationVersion() ) );
					}

					if ( currentProfile->getUseLocalPort() ) {
						ConnectProcess->addArgument( "--local-port" );
						ConnectProcess->addArgument( QString().setNum( currentProfile->getLocalPort() ) );
						appendLogEntry ( i18n( "Using userdefinied local port \"%1\"." ).arg( QString().setNum( currentProfile->getLocalPort() ) ) );
					}

					if ( currentProfile->getUseNtDomainName() ) {
						ConnectProcess->addArgument( "--domain" );
						ConnectProcess->addArgument( currentProfile->getNtDomainName() );
						appendLogEntry ( i18n( "Using (NT) domain name \"%1\"." ).arg( currentProfile->getNtDomainName() ) );
					}

					if ( currentProfile->getUsePerfectForwardSecrety() ) {
						ConnectProcess->addArgument( "--pfs" );
						ConnectProcess->addArgument( currentProfile->getPerfectForwardSecrety() );
						appendLogEntry ( i18n( "Using userdefinied perfect forward secrety \"%1\"." ).arg( currentProfile->getPerfectForwardSecrety() ) );
					}

					if ( currentProfile->getUseIkeGroup() ) {
						ConnectProcess->addArgument( "--dh" );
						ConnectProcess->addArgument( currentProfile->getIkeGroup() );
						appendLogEntry ( i18n( "Using userdefinied IKE group \"%1\"." ).arg( currentProfile->getIkeGroup() ) );
					}

					if ( currentProfile->getUseSingleDes() ) {
						ConnectProcess->addArgument( "--enable-1des" );
						appendLogEntry ( i18n( "Using single DES." ) );
					}

				}
			}

			connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
			connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );

			appendLogEntry ( i18n( "Trying to connect to server \"%1\" with user \"%2\" and IPSec ID \"%3\"...\n" ).arg( currentProfile->getGateway() ). arg( currentProfile->getUserName() ).arg( currentProfile->getID() ) );
			slotStatusMsg( i18n( "Connecting..." ), ID_CONNECTION_MSG );

			if ( !ConnectProcess->start() ) {
				KMessageBox::sorry( this, i18n( "Unable to start process (vpnc)!" ) );
				disconnectClicked();
			} else {
				appendLogEntry ( i18n( "vpnc started.\n" ) );

				connect ( ConnectProcess, SIGNAL( processExited () ), this, SLOT( vpncStarted() ) );

				connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
				timer.start( tryConnectTimeout * 1000, FALSE );

				mw->buttonConnect->setEnabled( false );

			}
		}
	} // vpnc

	/* ============ racoon ================ */
	else if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::racoon ) {
		//appendLogEntry ( "racoon not implemented yet.\n" );

		// looking for racoon
		if ( useDefaultPaths || pathToRacoon.isEmpty() ) {
			QFile vpnc( "/usr/sbin/racoon" );
			if ( vpnc.exists() )
				pathToRacoon = "/usr/sbin/racoon";
			else {
				vpnc.setName( "/usr/local/sbin/racoon" );
				if ( vpnc.exists() )
					pathToRacoon = "/usr/local/sbin/racoon";
				else {
					vpnc.setName( "/usr/bin/racoon" );
					if ( vpnc.exists() )
						pathToRacoon = "/usr/bin/racoon";
					else {
						KMessageBox::error( this, i18n( "Unable to find racoon!" ) );
						ok = false;
					}
				}
			}
		}

		// looking for setkey
		if ( useDefaultPaths || pathToSetkey.isEmpty() ) {
			QFile vpnc( "/usr/sbin/setkey" );
			if ( vpnc.exists() )
				pathToSetkey = "/usr/sbin/setkey";
			else {
				vpnc.setName( "/usr/local/sbin/setkey" );
				if ( vpnc.exists() )
					pathToSetkey = "/usr/local/sbin/setkey";
				else {
					vpnc.setName( "/usr/bin/setkey" );
					if ( vpnc.exists() )
						pathToSetkey = "/usr/bin/setkey";
					else {
						KMessageBox::error( this, i18n( "Unable to find setkey!" ) );
						ok = false;
					}
				}
			}
		}

		// here we go
		if ( ok ) {
			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface;
			if ( currentProfile->getNetworkDevice() == "default" ) {
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			} else {
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() )
				tmpInterface = "lo"; // no default interface found :(

			if ( LocalIP.isEmpty() )
				LocalIP = "127.0.0.1"; // no IP for default interface found :(

			mw->DebugOutput->append( i18n( "default interface: %1" ).arg( tmpInterface ) );
			mw->DebugOutput->append( i18n( "Local IP: %1" ).arg( LocalIP ) );

			QString Gateway = currentProfile->getGateway();
			QString RemoteNet = currentProfile->getRemoteNet();

			tmpPath = locateLocal ( "data", "kvpnc/" ); // should be
			//	KTempDir tmpdir;
			//	tmpPath = tmpdir.name();
			mw->DebugOutput->append( "tmppath: " + tmpPath );
			QString ProfileName = currentProfile->getName();
			QString CertPath = currentProfile->getCertPath();

			// write setkey.conf
			QFile file( tmpPath + "setkey." + ProfileName + ".conf" );
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) ) {
				stream << "spdadd " + LocalIP + " " + Gateway + " any -P out ipsec esp/tunnel/" + LocalIP + "-" + Gateway + "/require;" << "\n";
				stream << "spdadd " + Gateway + " " + LocalIP + " any -P in ipsec esp/tunnel/" + Gateway + "-" + LocalIP + "/require;" << "\n";
				stream << "spdadd " + LocalIP + " " + RemoteNet + " any -P out ipsec esp/tunnel/" + LocalIP + "-" + Gateway + "/require;" << "\n";
				stream << "spdadd " + RemoteNet + " " + LocalIP + " any -P in ipsec esp/tunnel/" + Gateway + "-" + LocalIP + "/require;" << "\n";
			}
			file.close();

			// write racoon.<profile>.conf
			file.setName( tmpPath + "racoon." + ProfileName + ".conf" );
			if ( file.open( IO_WriteOnly ) ) {
				stream.setDevice( &file );
				// 				stream2 << "# generated by kvpnc. Do not edit it." << "\n";
				// 				stream2 << "# profile: " + ProfileName << "\n";

				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "path certificate \"" + currentProfile->getCertPath() + "\";" << "\n";

				if ( currentProfile->getAuthType() == VpnAccountData::psk )
					stream << "path pre_shared_key \"" + tmpPath + "psk." + ProfileName + ".key" + "\";" << "\n";

				stream << "" << "\n";
				stream << "remote " + Gateway + " {" << "\n";
				stream << "  exchange_mode main;" << "\n";

				if ( currentProfile->getAuthType() == VpnAccountData::cert ) {
					stream << "  certificate_type x509 \"" << currentProfile->getX509Certificate() << "\" \"" << currentProfile->getX509Certificate() + "\";" << "\n";
					stream << "  verify_cert on;" << "\n";
				}
				if ( currentProfile->getAuthType() == VpnAccountData::cert ) {
					stream << "  my_identifier asn1dn;" << "\n";
					stream << "  peers_identifier asn1dn;" << "\n";
				}
				stream << "  proposal {" << "\n";
				stream << "    encryption_algorithm 3des;" << "\n";
				stream << "    hash_algorithm md5;" << "\n";

				// x509
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "    authentication_method rsasig;" << "\n";
				if ( currentProfile->getAuthType() == VpnAccountData::psk )
					stream << "    authentication_method pre_shared_key;" << "\n";
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "    dh_group modp1024;" << "\n";
				else
					stream << "    dh_group 2;" << "\n";
				stream << "  }" << "\n";
				stream << "}" << "\n";
				stream << "" << "\n";
				stream << "sainfo address " << LocalIP << " any address " << currentProfile->getRemoteNet() << " any {" << "\n";
				stream << "  pfs_group modp768;" << "\n";
				stream << "  encryption_algorithm 3des;" << "\n";
				stream << "  authentication_algorithm hmac_md5;" << "\n";
				stream << "  compression_algorithm deflate;" << "\n";
				stream << "}" << "\n";
			}
			file.close();
			stream.unsetDevice();

			// write setkey.<profile>.sh
			file.setName( tmpPath + "setkey." + ProfileName + ".sh" );
			stream.setDevice( &file );
			if ( file.open( IO_WriteOnly )
			   ) {
				stream << "#!/bin/sh" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "# Flush rules" << "\n";
				// 				stream << "#  AH" << "\n";
				stream << pathToSetkey << " -F" << "\n";
				// 				stream << "#  ESP" << "\n";
				stream << pathToSetkey << " -PF" << "\n";
				stream << "" << "\n";
				stream << "# run setkey" << "\n";
				stream << pathToSetkey << " -f " << tmpPath + "setkey." + currentProfile->getName() + ".conf" << "\n";
			}
			file.close();
			stream.unsetDevice();

			// iptables need the right interface (eth0:1 => eth0)
			QString tmpInterface2 = "";
			int pointPos = tmpInterface.find( ':', 0, FALSE );
			if ( pointPos > 1 ) {
				tmpInterface2 = tmpInterface.left( pointPos );
			} else
				tmpInterface2 = tmpInterface;

			// write iptables.<profile>.sh
			file.setName( tmpPath + "iptables." + ProfileName + ".add_ipsec_racoon.sh" );
			stream.setDevice( &file );
			if ( file.open( IO_WriteOnly )
			   ) {
				stream << "#!/bin/sh" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "#  AH" << "\n";
				stream << pathToIptables << " -I INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << pathToIptables << " -I OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  ESP" << "\n";
				stream << pathToIptables << " -I INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << pathToIptables << " -I OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  IKE" << "\n";
				stream << pathToIptables << " -I INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  NAT-T" << "\n";
				stream << pathToIptables << " -I INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";

			}
			file.close();
			stream.unsetDevice();

			if ( currentProfile->getAuthType() == VpnAccountData::psk ) {
				// write iptables.<profile>.sh
				file.setName( tmpPath + "psk." + ProfileName + ".key" );
				stream.setDevice( &file );
				if ( file.open( IO_WriteOnly ) ) {
					stream << "# generated by kvpnc. Do not edit it." << "\n";
					stream << "# profile: " + ProfileName << "\n";
					stream << currentProfile->getGateway() + "\t" + currentProfile->getPreSharedKey() << "\n";
					file.close();
					stream.unsetDevice();
				}
			}

			ConnectProcess = new QProcess ( this );
			//
			ConnectProcess->addArgument( "/bin/sh" );
			ConnectProcess->addArgument( tmpPath + "/setkey." + currentProfile->getName() + ".sh" );

			connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
			connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteRacoon() ) );

			if ( !ConnectProcess->start() ) {
				KMessageBox::error( this, i18n( "setkey.%1.sh start failed!" ).arg( ProfileName ) );
				disconnectClicked();
			}
			else {
				mw->buttonConnect->setEnabled(false);
			}
		}
	} // racoon

	/* ============ freeswan ================ */
	else if ( currentProfile != 0 && currentProfile->getConnectionType() == VpnAccountData::freeswan ) {
		//		appendLogEntry ( "freeswan not implemented yet.\n" );

		// looking for ipsec
		if ( useDefaultPaths || pathToFreeswan.isEmpty() ) {
			QFile vpnc( "/usr/sbin/ipsec" );
			if ( vpnc.exists() )
				pathToRacoon = "/usr/sbin/ipsec";
			else {
				vpnc.setName( "/usr/local/sbin/ipsec" );
				if ( vpnc.exists() )
					pathToRacoon = "/usr/local/sbin/ipsec";
				else {
					vpnc.setName( "/usr/bin/ipsec" );
					if ( vpnc.exists() )
						pathToRacoon = "/usr/bin/ipsec";
					else {
						KMessageBox::error( this, i18n( "Unable to find freeswan (ipsec)!" ) );
						ok = false;
					}
				}
			}
		}

		QString leftid = "";
		QString rightid = "";

		// x509
		if ( currentProfile->getAuthType() == VpnAccountData::cert ) {
			leftid = getX509CertificateID( currentProfile->getX509Certificate(), true ); // local
			if ( currentProfile->getUseSpecialRemoteID() )
				rightid = currentProfile->getSpecialRemoteID();
			else
				rightid = getX509CertificateID( currentProfile->getX509Certificate(), false ); // remote

			if ( leftid.isEmpty() || rightid.isEmpty() ) {
				KMessageBox::error( this, i18n( "Left or right ID is empty!" ) );
				ok = false;
			}
		}


		// 		// here we go
		if ( ok ) {
			QString tmpInterface;
			QString LocalIP;
			NetworkInterface iface;
			if ( currentProfile->getNetworkDevice() == "default" ) {
				tmpInterface = iface.getDefaultInterface();
				LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
			} else {
				tmpInterface = currentProfile->getNetworkDevice();
				LocalIP = iface.getInterfaceIP( currentProfile->getNetworkDevice() );
			}
			if ( tmpInterface.isEmpty() )
				tmpInterface = "lo"; // no default interface found :(

			if ( LocalIP.isEmpty() )
				LocalIP = "127.0.0.1"; // no IP for default interface found :(

			mw->DebugOutput->append( "default interface: " + tmpInterface );
			mw->DebugOutput->append( "LocalIP: " + LocalIP );

			QString Gateway = currentProfile->getGateway();
			QString RemoteNet = currentProfile->getRemoteNet();

			tmpPath = locateLocal ( "data", "kvpnc/" ); // should be
			//	KTempDir tmpdir;
			//	tmpPath = tmpdir.name();
			mw->DebugOutput->append( "tmppath: " + tmpPath );
			QString ProfileName = currentProfile->getName();
			QString CertPath = currentProfile->getCertPath();
			QString PrivateKeyPass = currentProfile->getPrivateKeyPass();
			QString PrivateKey = currentProfile->getPrivateKey();

			// write /etc/ipsec.secrets
			//QFile file( tmpPath + "ipsec." + ProfileName + ".secrets" );
			QFile file( "/etc/ipsec.secrets" ); // warning: static
			QTextStream stream( &file );
			if ( file.open( IO_WriteOnly ) ) {
				// cert
				if ( currentProfile->getAuthType() == VpnAccountData::cert ) {
					stream << ": RSA " + PrivateKey + " \"" + PrivateKeyPass + "\"\n";
					//		mw->DebugOutput->append( "cert: private key: : RSA " + PrivateKey + " \"" + PrivateKeyPass );
				}
				// PSK
				else {
					stream << ": RSA " + PrivateKey + "\n";
					//		mw->DebugOutput->append( "cert: private key: "+PrivateKey);
				}

			}
			file.close();

			// write /etc/ipsec.conf
			//file.setName( tmpPath + "freeswan." + ProfileName + ".conf" );
			file.setName( "/etc/ipsec.conf" ); // warning: static
			if ( file.open( IO_WriteOnly ) ) {
				stream.setDevice( &file );
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "config setup" << "\n";
				stream << "    interfaces=%defaultroute" << "\n";
				stream << "    klipsdebug=none" << "\n";
				stream << "    plutodebug=none" << "\n";
				stream << "    plutoload=%search" << "\n";
				stream << "    plutostart=%search" << "\n";
				stream << "" << "\n";
				stream << "conn %default" << "\n";
				stream << "    authby=rsasig" << "\n";

				// x509
				if ( currentProfile->getAuthType() == VpnAccountData::cert ) {
					stream << "    leftrsasigkey=%cert" << "\n";
					stream << "    rightrsasigkey=%cert" << "\n";
				}
				if ( currentProfile->getAuthType() == VpnAccountData::psk ) {
					stream << "    leftrsasigkey=" << currentProfile->getPrivateKey() << "\n";
					stream << " rightrsasigkey = " << currentProfile->getPreSharedKey() << "\n";
				}
				stream << "    left=%defaultroute" << "\n";
				stream << "    keyingtries=1" << "\n";
				stream << "    #keylife=1200s" << "\n";
				stream << "    #ikelifetime=1200s" << "\n";
				stream << "" << "\n";
				stream << "conn roadwarrior-to-gw" << "\n";
				stream << "    auto=start" << "\n";
				stream << "    leftsubnet=" << "\n";
				stream << "    rightnexthop=" << "\n";
				//				stream << "    rightnexthop=" << "192.168.1.1"  << "\n";
				if ( currentProfile->getAuthType() == VpnAccountData::cert )
					stream << "    leftcert=" + currentProfile->getX509Certificate() << "\n";
				stream << "    leftid = \"" + leftid + "\"" << "\n";
				stream << "    right=" + currentProfile->getGateway() << "\n";
				stream << "    rightid=\"" + rightid + "\"" << "\n";
				//stream << "    rightid=\"/C=DE/ST=Sachsen-Anhalt/O=Powersoft/OU=IT/CN=212.100.100.212\"" << "\n";

			}

			stream << "  rightsubnet=" << currentProfile->getRemoteNet() << "\n";

			file.close();
			stream.unsetDevice();

			// iptables need the right interface (eth0:1 => eth0)
			QString tmpInterface2 = "";
			int pointPos = tmpInterface.find( ':', 0, FALSE );
			if ( pointPos > 1 ) {
				tmpInterface2 = tmpInterface.left( pointPos );
			} else
				tmpInterface2 = tmpInterface;

			// write iptables.<profile>_add_ipsec_freeswan.sh
			file.setName( tmpPath + "iptables." + ProfileName + ".add_ipsec_freeswan.sh" );
			stream.setDevice( &file );
			if ( file.open( IO_WriteOnly ) ) {
				stream << "#!/bin/sh" << "\n";
				stream << "# generated by kvpnc. Do not edit it." << "\n";
				stream << "# profile: " + ProfileName << "\n";
				stream << "" << "\n";
				stream << "#  AH" << "\n";
				stream << pathToIptables << " -I INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << pathToIptables << " -I OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  ESP" << "\n";
				stream << pathToIptables << " -I INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << pathToIptables << " -I OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  IKE" << "\n";
				stream << pathToIptables << " -I INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				stream << "" << "\n";
				stream << "#  NAT-T" << "\n";
				stream << pathToIptables << " -I INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";

			}
			file.close();
			stream.unsetDevice();

			if ( currentProfile->getAuthType() == VpnAccountData::psk ) {
				// write iptables.<profile>.sh
				file.setName( tmpPath + "psk." + ProfileName + ".key" );
				stream.setDevice( &file );
				if ( file.open( IO_WriteOnly ) ) {
					stream << currentProfile->getPreSharedKey() << "\n";
				}
				file.close();
				stream.unsetDevice();
			}

			ConnectProcess = new QProcess ( this );
			//
			ConnectProcess->addArgument( pathToFreeswan );
			ConnectProcess->addArgument( "setup" );
			ConnectProcess->addArgument( "start" );

			connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
			connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteFreeswan() ) );

			if ( !ConnectProcess->start() ) {
				KMessageBox::error( this, i18n( "freeswan (ipsec) start failed!" ).arg( ProfileName ) );
				disconnectClicked();
			} else {

				connectionTime.start();
				connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
				durationTimer.start( 1000, FALSE );
				setGuiStatus ( connected );
				isConnected = true;
				if ( currentProfile->getDoPingIP() )
					sleep( 3 );
				doPingTest();
			}


		}
	} // freeswan

	// unsupported
	else {
		appendLogEntry ( i18n( "Unsupported connection type.\n" ) );

	}
}


void KVpnc::disconnectClicked() {
	if ( isConnected ) {
		QString type_string = "";
		bool ok = true; // if ok we will run process

		// here we go
		if ( ok ) {
			if ( currentProfile->getConnectionType() == VpnAccountData::cisco ) {

				tmpPath = locateLocal ( "data", "kvpnc/" );
				//Vpnc_pid_file = tmpPath + "vpnc-pid." + currentProfile->getID() + ".pid"; // vpnc-pid.<id>.pid
				VpncDisconnectScript = tmpPath + "vpnc-disconect." + currentProfile->getID() + ".sh";
				// write vpnc connect script
				QFile pidFile( Vpnc_pid_file );
				QTextStream pidFileStream( &pidFile );
				QString Vpnc_pid = "";
				if ( pidFile.open( IO_ReadOnly ) ) {
					Vpnc_pid = pidFileStream.readLine();
				}

				QFile file( VpncDisconnectScript );
				QTextStream stream( &file );
				if ( file.open( IO_WriteOnly ) ) {
					//					stream << "#!/bin/bash\n";
					stream << "# generated by kvpnc. Do not edit it." << "\n";
					stream << "# profile: " + currentProfile->getID() << "\n";
					stream << "" << "\n";

					if ( !Vpnc_pid.isEmpty() ) {
						if ( KvpncDebugLevel > 0 )
							appendLogEntry( "debug: " + i18n( "Vpnc pid file found, killing process %1" ).arg( Vpnc_pid ) );
						stream << "/usr/bin/kill -3 " << Vpnc_pid << "\n";
					} else {
						if ( KvpncDebugLevel > 0 )
							appendLogEntry( "debug: " + i18n( "No vpnc pid file found, using \"killall\" for killing vpnc." ) );
						stream << "/usr/bin/killall -3 kvpnc" << "\n";
					}

					stream << "ip route del default > /dev/null 2>&1\n";
					stream << "ip route add `" + OldDefaultroute + "`\n";
					stream << "ip route del < " + VpncGateway + "\n";
					stream << "ip route flush cache\n";
				}
				file.close();

				DisconnectProcess = new QProcess ();
				DisconnectProcess->addArgument( "/bin/sh" );
				DisconnectProcess->addArgument( VpncDisconnectScript );

				if ( VpncDebugLevel > 0 ) {
					appendLogEntry ( i18n( "Setting vpnc debug level %1." ).arg( QString().setNum( VpncDebugLevel ) ) );
					DisconnectProcess->addArgument( "--debug" );
					DisconnectProcess->addArgument( QString().setNum( VpncDebugLevel ) );
				}

				connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_disconnect() ) );
				connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_disconnect() ) );

				type_string = "vpnc_disconnect";

				if ( !DisconnectProcess->start() ) {
					KMessageBox::sorry( this, i18n( "Unable to start process (%s)!" ).arg( type_string ) );
				} else {
					connectionEnded();
				}
			}

			// racoon
			if ( currentProfile->getConnectionType() == VpnAccountData::racoon ) {

				// 				// looking for setkey
				// 				if ( pathToSetkey.isEmpty() ) {
				// 					QFile vpnc( "/usr/sbin/setkey" );
				// 					if ( vpnc.exists() )
				// 						pathToSetkey = "/usr/sbin/setkey";
				// 					else {
				// 						vpnc.setName( "/usr/local/sbin/setkey" );
				// 						if ( vpnc.exists() )
				// 							pathToSetkey = "/usr/local/sbin/setkey";
				// 						else {
				// 							vpnc.setName( "/usr/bin/setkey" );
				// 							if ( vpnc.exists() )
				// 								pathToSetkey = "/usr/bin/setkey";
				// 							else {
				// 								KMessageBox::error( this, i18n( "Unable to find setkey!" ) );
				// 								ok = false;
				// 							}
				// 						}
				// 					}
				// 				}

				// iptables need the right interface (eth0:1 => eth0)
				QString tmpInterface2 = "";
				int pointPos = tmpInterface.find( ':', 0, FALSE );
				if ( pointPos > 1 ) {
					tmpInterface2 = tmpInterface.left( pointPos );
				} else
					tmpInterface2 = tmpInterface;

				QFile file1( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_racoon.sh" );
				QTextStream	stream1( &file1 );
				if ( file1.open( IO_WriteOnly ) ) {
					stream1 << "#!/bin/sh" << "\n";
					stream1 << "# generated by kvpnc. Do not edit it." << "\n";
					stream1 << "# profile: " + currentProfile->getName() << "\n";
					stream1 << "" << "\n";
					stream1 << "#  AH" << "\n";
					stream1 << pathToIptables << " -D INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << pathToIptables << " -D OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  ESP" << "\n";
					stream1 << pathToIptables << " -D INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << pathToIptables << " -D OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  IKE" << "\n";
					stream1 << pathToIptables << " -D INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  NAT-T" << "\n";
					stream1 << pathToIptables << " -D INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				}
				file1.close();

				QFile file2( tmpPath + "setkey." + currentProfile->getName() + ".remove_setkey.sh" );
				QTextStream	stream2( &file2 );
				if ( file2.open( IO_WriteOnly ) ) {
					stream2 << "#!/bin/sh" << "\n";
					stream2 << "# generated by kvpnc. Do not edit it." << "\n";
					stream2 << "# profile: " + currentProfile->getName() << "\n";
					stream2 << pathToSetkey << "-PF" << "\n";
					stream2 << pathToSetkey << "-F" << "\n";
				}
				file2.close();

				//if ( RacoonProcess != 0 )
				terminateConnection( *RacoonProcess );
				
				removeSetkey();

// 				DisconnectProcess->clearArguments();
// 				DisconnectProcess->addArgument( "/usr/bin/killall" );
// 				DisconnectProcess->addArgument( "-3" );
// 				DisconnectProcess->addArgument( "racoon" );
// 
// 				connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
// 				connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
// 				connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeSetkey() ) );
// 
// 				if ( !DisconnectProcess->start() )
// 					KMessageBox::error( this, i18n( "\"killall -3 racoon\" start failed!" ) );
// 				else {
// 					TailProcess->tryTerminate();
// 					if ( KvpncDebugLevel > 0 )
// 						appendLogEntry( "debug: " + i18n( "\"killall -3 racoon\" successful." ) );
// 				}

			}

			// freeswan
			if ( currentProfile->getConnectionType() == VpnAccountData::freeswan ) {
				type_string = "freeswan";

				// 				// looking for ipsec
				// 				if ( pathToFreeswan.isEmpty() ) {
				// 					QFile vpnc( "/usr/sbin/ipsec" );
				// 					if ( vpnc.exists() )
				// 						pathToRacoon = "/usr/sbin/ipsec";
				// 					else {
				// 						vpnc.setName( "/usr/local/sbin/ipsec" );
				// 						if ( vpnc.exists() )
				// 							pathToRacoon = "/usr/local/sbin/ipsec";
				// 						else {
				// 							vpnc.setName( "/usr/bin/ipsec" );
				// 							if ( vpnc.exists() )
				// 								pathToRacoon = "/usr/bin/ipsec";
				// 							else {
				// 								KMessageBox::error( this, i18n( "Unable to find freeswan (ipsec)!" ) );
				// 								ok = false;
				// 							}
				// 						}
				// 					}
				// 				}

				// iptables need the right interface (eth0:1 => eth0)
				QString tmpInterface2 = "";
				int pointPos = tmpInterface.find( ':', 0, FALSE );
				if ( pointPos > 1 ) {
					tmpInterface2 = tmpInterface.left( pointPos );
				} else
					tmpInterface2 = tmpInterface;

				QFile file1( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_freeswan.sh" );
				QTextStream	stream1( &file1 );
				if ( file1.open( IO_WriteOnly ) ) {
					stream1 << "#!/bin/sh" << "\n";
					stream1 << "# generated by kvpnc. Do not edit it." << "\n";
					stream1 << "# profile: " + currentProfile->getName() << "\n";
					stream1 << "" << "\n";
					stream1 << "#  AH" << "\n";
					stream1 << pathToIptables << " -D INPUT -p 50 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << pathToIptables << " -D OUTPUT -p 50 -d " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  ESP" << "\n";
					stream1 << pathToIptables << " -D INPUT -p 51 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << pathToIptables << " -D OUTPUT -p 51 -d " << currentProfile->getGateway() << " -j  ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  IKE" << "\n";
					stream1 << pathToIptables << " -D INPUT -p udp --dport 500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
					stream1 << "" << "\n";
					stream1 << "#  NAT-T" << "\n";
					stream1 << pathToIptables << " -D INPUT -p udp --dport 4500 -i " << tmpInterface2 << " -s " << currentProfile->getGateway() << " -j ACCEPT" << "\n";
				}
				file1.close();

				DisconnectProcess = new QProcess ( this );
				DisconnectProcess->addArgument( pathToFreeswan );
				DisconnectProcess->addArgument( "setup" );
				DisconnectProcess->addArgument( "stop" );

				connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
				connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
				connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteFreeswan() ) );

				if ( !DisconnectProcess->start() )
					KMessageBox::error( this, i18n( "start of \"ipsec setup stop\" failed." ) );
			} else {
				if ( KvpncDebugLevel > 0 )
					appendLogEntry( "debug: " + i18n( "starting \"ipsec setup stop\" was successful." ) );
				connectionEnded();
				setGuiStatus( disconnected );
			}
		}
	}
	//else
	//appendLogEntry ( i18n( "Not connected.\n" ) );

}

void KVpnc::helpClicked() {
	/*
	mw->DebugOutput->append( "getting userID:\n" );
	QString leftid = getX509CertificateID( "/etc/racoon/certs/mykeys_thielecke.pem", true ); // we
	mw->DebugOutput->append( "getting issuerID:\n" );
	QString rightid = getX509CertificateID( "/etc/racoon/certs/mykeys_thielecke.pem", false ); // remote
	*/ 
	//doPingTest();


/*
		HelpDialog d1;
		d1.exec();

		slotStatusMsg ( i18n( "Help dialog called." ), ID_FLASH_MSG );
*/
	
	/* network tests */
	/*
	NetworkInterface iface;
	bool success = iface.interfaceExist( "eth0" );
	QString succString;
	if ( success == true )
	mw->DebugOutput->append( "Testing interface eth0: success" );
				else
				mw->DebugOutput->append( "Testing interface eth0: failed" );

					QStringList list = iface.getAllNetworkInterfaces();

					for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
					QString aa = QString( "Interface: " + *it );
					mw->DebugOutput->append( aa );
				}

					QString defaultinterface = "Defaultinterface: " + iface.getDefaultInterface();
					if ( defaultinterface.isEmpty() )
					defaultinterface += "none";

					mw->DebugOutput->append( defaultinterface );


					QString interfaceaddress = "Interface adress: " + iface.getInterfaceAddress( "127.0.0.1" );
					if ( interfaceaddress.isEmpty() )
					interfaceaddress += "none";
					mw->DebugOutput->append( interfaceaddress );

					QString interfaceip = "Interface IP: " + iface.getInterfaceIP( "eth0:1" );
					if ( interfaceip.isEmpty() )
					interfaceip += "none";
					mw->DebugOutput->append( interfaceip );
					*/
}

void KVpnc::readFromStderr_vpnc() {
	//mw->DebugOutput->append( "readFromStderr_vpnc!\n" );
	//	while ( ConnectProcess->canReadLineStderr() ) {
	//		ProcessMsg_connect = ConnectProcess->readLineStderr();
	ProcessMsg_connect = QString( ConnectProcess->readStderr() );

	if ( !ProcessMsg_connect.isEmpty() )
		// 			appendLogEntry ( ProcessMsg_connect + "\n" );
		appendLogEntry ( ProcessMsg_connect );

	bool connectSuccess = false;
	if ( ProcessMsg_connect.find( "started in background", 0, FALSE ) > -1 )
		connectSuccess = true;

	if ( ProcessMsg_connect.find( "started in foreground", 0, FALSE ) > -1 )
		connectSuccess = true;

	if ( connectSuccess ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry( "debug: " + i18n( "Connection established." ) );

		//connectionTime.restart();
		//uint duration = connectionTime.elapsed();

		sec = hour = min = 0;

		isConnected = true;
		timer.stop();

		connectionTime.start();
		connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
		durationTimer.start( 1000, FALSE );
		setGuiStatus ( connected );
		connectSuccess = false;
	}

	bool abort = false;

	//ProcessMsg_connect = QString(ConnectProcess->readStdout());
	if ( this->VpncDebugLevel > 0 )
		this->appendLogEntry ( QString ( "raw stderr: " + ProcessMsg_connect ) );


	// auth failed
	if ( ProcessMsg_connect.find( "AUTHENTICATION_FAILED", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Authentication failed!" ) );
		abort = true;
	}

	if ( ProcessMsg_connect.find( "authentication unsuccessful", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Authentication failed!" ) );
		abort = true;
	}

	// no answer
	if ( ProcessMsg_connect.find( "no response from target", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "No Response from VPN server" ) );
		abort = true;
	}

	// initalizng tunnel interface failed
	if ( ProcessMsg_connect.find( "can't initialise tunnel interface", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Tunnel interface can't be initalized" ) );
		abort = true;
	}

	// host unknown
	if ( ProcessMsg_connect.find( "unknown host" , 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Host unknown" ) );
		abort = true;
	}

	//socket creation failed
	if ( ProcessMsg_connect.find( "making socket", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Socket creation failed" ) );
		abort = true;
	}


	// binding to port failed
	if ( ProcessMsg_connect.find( "binding to port", 0 , FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Port binding failed" ) );
		abort = true;
	}


	if ( ProcessMsg_connect.find( "Network is unreachable", 0 , FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "No network reachable" ) );
		terminateConnection ( *ConnectProcess );
	}

	if ( ProcessMsg_connect.find( "vpnc found running", 0 , FALSE ) > -1 ) {
		//KMessageBox::sorry( this, i18n( "vpnc is already running, kill it manually" ) );
		//terminateConnection ( *ConnectProcess );
		if ( KvpncDebugLevel > 0 ) {
			appendLogEntry ( "debug: " + i18n( "Vpnc found running, killing it...\n" ) );
		}
		disconnectClicked();
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "Trying again...\n" ) );
		connectClicked();
	}

	if ( abort ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry( "debug: " + i18n( "There is a reason for stop connecting, terminating \"vpnc\" process." ) );
		terminateConnection ( *ConnectProcess );
	}

	//	}
}

void KVpnc::readFromStdout_vpnc() {
	//mw->DebugOutput->append( "readFromStdout_vpnc!\n" );

	//while ( ConnectProcess->canReadLineStdout() ) {
	//	ProcessMsg_connect = ConnectProcess->readLineStdout();
	ProcessMsg_connect = QString( ConnectProcess->readStdout() );
	if ( !ProcessMsg_connect.isEmpty() )
		// 			appendLogEntry ( ProcessMsg_connect + "\n" );
		appendLogEntry ( ProcessMsg_connect );

	bool connectSuccess = false;
	if ( ProcessMsg_connect.find( "started in background", 0, FALSE ) > -1 )
		connectSuccess = true;

	if ( ProcessMsg_connect.find( "started in foreground", 0, FALSE ) > -1 )
		connectSuccess = true;

	// success
	// background -> normal but no check if running :(
	// foreground -> stays in foreground and we can test if running :)
	if ( connectSuccess ) {

		if ( KvpncDebugLevel > 0 )
			appendLogEntry( "debug: " + i18n( "Connection established." ) );

		//connectionTime.restart();
		//uint duration = connectionTime.elapsed();

		sec = hour = min = 0;

		isConnected = true;
		timer.stop();

		connectionTime.start();
		connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
		durationTimer.start( 1000, FALSE );
		setGuiStatus ( connected );
		connectSuccess = false;
	}

	//ProcessMsg_connect = QString(ConnectProcess->readStdout());

	if ( this->VpncDebugLevel > 0 )
		this->appendLogEntry ( QString ( "raw stdout: " + ProcessMsg_connect ) );

	if ( deviceaddrstr2addr ) {
		deviceaddrstr = ProcessMsg_connect;
		// 			this->appendLogEntry ( QString ( "deviceaddrstr: " + deviceaddrstr ) );
		deviceaddr = "addr: " + QString( deviceaddrstr.simplifyWhiteSpace ().section( ' ', 1, 1 ) ).section( ':', 1, 1 );
		// 			this->appendLogEntry ( deviceaddr ); //
		deviceaddrstr2addr = false;
		disconnect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
		disconnect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );
	}

	// requesting password
	if ( ProcessMsg_connect.find( "Enter password", 0, FALSE ) > -1 ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "User password requested, send it...\n" ) );
		QString UserPassStr = currentProfile->getUserPassword();
		ConnectProcess->writeToStdin( UserPassStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting password
	if ( ProcessMsg_connect.find( "Enter IPSec secret", 0, FALSE ) > -1 ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "Group password requested, send it...\n" ) );
		QString GrpPassStr = currentProfile->getGroupPassword();
		ConnectProcess->writeToStdin( GrpPassStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting gatway
	if ( ProcessMsg_connect.find( "Enter IPSec gateway address", 0, FALSE ) > -1 ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "IPSec gateway address requested, send it...\n" ) );
		QString GatewayStr = currentProfile->getGateway();
		ConnectProcess->writeToStdin( GatewayStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting IPSec ID
	if ( ProcessMsg_connect.find( "Enter IPSec ID", 0, FALSE ) > -1 ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "IPSec ID requested, send it...\n" ) );
		QString IpsecIDStr = currentProfile->getID();
		ConnectProcess->writeToStdin( IpsecIDStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	// requesting username
	if ( ProcessMsg_connect.find( "Enter username", 0, FALSE ) > -1 ) {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "Username requested, send it...\n" ) );
		QString UsernameStr = currentProfile->getUserName();
		ConnectProcess->writeToStdin( UsernameStr );
		ConnectProcess->writeToStdin( "\n" );
	}

	if ( ProcessMsg_connect.find( "vpnc found running", 0 , FALSE ) > -1 ) {
		//KMessageBox::sorry( this, i18n( "vpnc is already running, kill it manually" ) );
		//terminateConnection ( *ConnectProcess );
		if ( KvpncDebugLevel > 0 ) {
			appendLogEntry ( "debug: " + i18n( "Vpnc found running, killing it...\n" ) );
		}
		disconnectClicked();
		if ( KvpncDebugLevel > 0 )
			appendLogEntry ( "debug: " + i18n( "Trying again...\n" ) );
		connectClicked();
	}

	slotStatusMsg ( i18n( "Ready." ), ID_STATUS_MSG );
	//	}
}

void KVpnc::readFromStderr_disconnect() {

	ProcessMsg_disconnect = DisconnectProcess->readLineStderr();
	if ( !ProcessMsg_disconnect.isEmpty() )
		appendLogEntry ( ProcessMsg_disconnect + "\n" );
}

void KVpnc::readFromStdout_disconnect() {
	ProcessMsg_disconnect = DisconnectProcess->readLineStdout();

	if ( !ProcessMsg_disconnect.isEmpty() )
		appendLogEntry ( ProcessMsg_disconnect + "\n" );
}

void KVpnc::readFromStdout_racoon() {
	//while ( ConnectProcess->canReadLineStdout() ) {
	//		ProcessMsg_connect = ConnectProcess->readLineStdout();
	QString ProcessMsg_connect = QString( ConnectProcess->readStdout() );

	if ( pingtest ) {
		// 		if ( KvpncDebugLevel > 0 )
		// 			mw->DebugOutput->append( "raw out (pingtest): " + ProcessMsg_connect );

		if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 ) {
			mw->DebugOutput->append( i18n( "out: success: <font color=\"green\">" ) + ProcessMsg_connect + "</font>" );
		} else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 ) {
			mw->DebugOutput->append( i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		} else {
			if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 ) {
				KMessageBox::error( this, i18n( "Binding interface failed!" ) );
			}
			if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 ) {
				KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
			}


			mw->DebugOutput->append( "out: " + ProcessMsg_connect );
		}
	}

	// normal
	else {
		mw->DebugOutput->append( "out: " + ProcessMsg_connect );
	}
	//}
}

void KVpnc::readFromStderr_racoon() {
	//	while ( ConnectProcess->canReadLineStderr() ) {
	//		ProcessMsg_connect = ConnectProcess->readLineStderr();
	QString ProcessMsg_connect = QString( ConnectProcess->readStderr() );
	mw->DebugOutput->append( "err: " + ProcessMsg_connect );

	if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "Binding interface failed!" ) );
	}

	if ( ProcessMsg_connect.find( "no default route, %defaultroute cannot cope!!!", 0, FALSE ) > -1 ) {
		KMessageBox::error( this, i18n( "No default route found, nesessary for freeswan!" ) );
		disconnectClicked();
	}

	if ( ProcessMsg_connect.find( "SIOCADDRT: File exists", 0, FALSE ) > -1 ) {
		mw->DebugOutput->append( i18n( "err: setting route failed: route still exists" ) );
	}

	if ( ProcessMsg_connect.find( "SIOCDELRT: File exists", 0, FALSE ) > -1 ) {
		mw->DebugOutput->append( i18n( "err: deleting route failed: route dont exists" ) );
	}

	if ( pingtest ) {
		//	mw->DebugOutput->append( "raw err (pingtest): " + ProcessMsg_connect );
		if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 ) {
			mw->DebugOutput->append( i18n( "out: success: <font color=\"green\">" ) + ProcessMsg_connect + "</font>" );
		} else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 ) {
			mw->DebugOutput->append( i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
		} else {
			if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 ) {
				KMessageBox::error( this, i18n( "Binding interface failed!" ) );
			}
			if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 ) {
				KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
			}
		}
	}
}

void KVpnc::readFromStdout_tail_racoon() {
	//QString ProcessMsg_connect = QString( TailProcess->readStdout() );
	while ( TailProcess->canReadLineStdout() ) {
		QString ProcessMsg_connect = TailProcess->readLineStdout();
		if ( KvpncDebugLevel > 0 ) {
			mw->DebugOutput->append( "debug racoon: <font color=\"orange\">" + ProcessMsg_connect+"</font>" );
		}
	}
}

void KVpnc::readFromStderr_tail_racoon() {
	//QString ProcessMsg_connect = QString( TailProcess->readStderr() );
	while ( TailProcess->canReadLineStdout() ) {
		QString ProcessMsg_connect = TailProcess->readLineStdout();
		if ( KvpncDebugLevel > 0 ) {
			mw->DebugOutput->append( "debug racoon (err): <font color=\"red\">"  + ProcessMsg_connect+"</font>" );
		}
	}
}

void KVpnc::readFromStdout_ping() {
	//while ( ConnectProcess->canReadLineStdout() ) {
	//ProcessMsg_connect = ConnectProcess->readLineStdout();
	ProcessMsg_connect = QString( PingProcess->readStdout() );

	//mw->DebugOutput->append( "raw out (pingtest): " + ProcessMsg_connect );

	if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 ) {
		mw->DebugOutput->append("ping "+ i18n( "out: success: <font color=\"darkgreen\">" ) + ProcessMsg_connect + "</font>" );
	} else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 ) {
		mw->DebugOutput->append( "ping "+i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
	} else {
		if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 ) {
			KMessageBox::error( this, i18n( "Binding interface failed!" ) );
		}
		if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 ) {
			KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
		}


		//mw->DebugOutput->append( "out: " + ProcessMsg_connect );
	}
	//	}
}

void KVpnc::readFromStderr_ping() {
	//	while ( ConnectProcess->canReadLineStderr() ) {
	//	ProcessMsg_connect = ConnectProcess->readLineStderr();
	ProcessMsg_connect = QString( PingProcess->readStderr() );

	//mw->DebugOutput->append( "raw err (pingtest): " + ProcessMsg_connect );
	if ( ProcessMsg_connect.find( "time=", 0, FALSE ) > -1 ) {
		mw->DebugOutput->append( "ping "+i18n( "out: success: <font color=\"darkgreen\">" ) + ProcessMsg_connect + "</font>" );
	} else if ( ProcessMsg_connect.find( "100 % packet loss", 0, FALSE ) > -1 ) {
		mw->DebugOutput->append( "ping "+i18n( "out: failed: <font color=\"red\">" ) + ProcessMsg_connect + "</font>" );
	} else {
		if ( ProcessMsg_connect.find( "failed to bind", 0, FALSE ) > -1 ) {
			KMessageBox::error( this, i18n( "Binding interface failed!" ) );
		}
		if ( ProcessMsg_connect.find( "Destination Host Unreachable", 0, FALSE ) > -1 ) {
			KMessageBox::error( this, i18n( "Destination host is unreachable!" ) );
		}
	}
	//}
}


void KVpnc::terminateConnection ( QProcess & proc ) {
	if ( proc.isRunning() ) {
		proc.tryTerminate();
		QTimer::singleShot( 5000, &proc, SLOT( kill() ) );
		//sleep(5000);
	}
	timer.stop();
	durationTimer.stop();
	isConnected = false;
	connectCounter = 0;
	setGuiStatus ( disconnected );

}

void KVpnc::connectionEnded() {
	durationTimer.stop();
	isConnected = false;

	setGuiStatus ( disconnected );

	appendLogEntry ( i18n( "Successful disconnected." ) );

	// clean up scripts
	if ( !holdGeneratedScripts ) {

		if ( currentProfile->getConnectionType() == VpnAccountData::racoon ) {
			QFile file( tmpPath + "setkey." + currentProfile->getName() + ".conf" );
			file.remove();
			file.setName( tmpPath + "/setkey." + currentProfile->getName() + ".sh" );
			file.remove();
			file.setName( tmpPath + "racoon." + currentProfile->getName() + ".conf" );
			file.remove();
			file.setName( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_racoon.sh" );
			file.remove();
			file.setName( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_racoon.sh" );
			file.remove();
			file.setName( tmpPath + "setkey." + currentProfile->getName() + ".remove_setkey.sh" );
			file.remove();

			if ( currentProfile->getAuthType() == VpnAccountData::psk ) {
				file.setName( tmpPath + "psk." + currentProfile->getName() + ".key" );
				file.remove();
			}
			file.setName( tmpPath + "racoon." + currentProfile->getName() + ".log" );
			file.remove();
		} // racoon

		if ( currentProfile->getConnectionType() == VpnAccountData::freeswan ) {
			QFile file( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_freeswan.sh" );
			file.remove();
			file.setName( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_freeswan.sh" );
			file.remove();

		}	// freeswan

	}

	// tmp
	QString hour_string = QString().sprintf( "%2d", hour );
	QString min_string = QString().sprintf( "%2d", min );
	QString sec_string = QString().sprintf( "%2d", sec );

	QString durationString ( i18n( "%1:%2:%3" ).arg( hour_string ).arg( min_string , sec_string ) );
	appendLogEntry ( i18n( "Connection duration was %1 hours, %2 minutes, %3 seconds" ).arg( hour_string, min_string , sec_string ) );
	connectCounter = 0;
}

void KVpnc::slotConnectTimedOut() {
	this->connectCounter++;

	if ( connectCounter > connectSleepRetry ) {
		// it dont get it connected so we kill it :)
		appendLogEntry ( i18n( "Timeout! Kill connect process!" ) );
		this->ConnectProcess->kill();
		timer.stop();
		isConnected = false;
		mw->buttonConnect->setEnabled( true );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( true );
		this->disconnectAction->setEnabled( false );
		//parent->statusBar() ->insertItem( i18n( "Connected" ), 2, 0, true );
		kickerMenu->setStatus( disconnected );
		appendLogEntry ( i18n( "Connection failed (timeout)." ) );
		disconnectClicked();
		slotStatusMsg( i18n( "Connection timed out, vpnc killed." ), ID_FLASH_MSG );
		slotStatusMsg( i18n( "Disconnected" ), ID_CONNECTION_MSG );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add
			( kickerMenu, i18n( "Disconnected" ) );
		connectCounter = 0;
	} else {
		appendLogEntry ( i18n( "Waiting %1 seconds for connect..." ).arg( connectSleepDelay ) );
		slotStatusMsg( i18n( "Waiting for connect..." ), ID_FLASH_MSG );
	}
}

void KVpnc::newSessionClicked() {
	// call new profile dialog...
	NewProfileDialog newdlg( &AccountList, mw->DebugOutput );
	newdlg.exec();
	if ( newdlg.newProfileCreated ) {
		QString Name = newdlg.profileData->getName();
		mw->SessionCombo->insertItem( Name );
		mw->SessionCombo->setCurrentText( Name );
		VpnAccountData * it;
		for ( it = AccountList.first(); it; it = AccountList.next() ) {
			if ( it->getName() == Name ) {
				currentProfile = it;
				break;
			}
		}
		slotStatusMsg ( i18n( "Profile \"%1\" added." ).arg( mw->SessionCombo->currentText() ), ID_FLASH_MSG );
		appendLogEntry ( i18n( "Profile \"%1\" added." ).arg( mw->SessionCombo->currentText() ) );
		saveOptions();
	} else {
		slotStatusMsg ( i18n( "Creating new profile canceled." ), ID_FLASH_MSG );
		appendLogEntry ( i18n( "Creating new profile canceled." ) );
	}
}

void KVpnc::saveSessionClicked() {
	if ( !mw->SessionCombo->currentText().isEmpty() ) {
		saveOptions();
	}
}

void KVpnc::deleteSessionClicked() {
	QString Name = currentProfile->getName();
	if ( !Name.isEmpty() ) {
		int result = KMessageBox::questionYesNo ( this, i18n( "Really delete \"%1\"?" ).arg( currentProfile->getName() ), i18n( "Delete?" ) );
		if ( result ) {
			mw->SessionCombo->removeItem( mw->SessionCombo->currentItem() );
			AccountList.remove( currentProfile );
			if ( !AccountList.isEmpty() ) {
				AccountList.first();
				mw->SessionCombo->setCurrentItem( 0 );
			} else {
				this->deleteSessionAction->setEnabled ( FALSE );
				//				this->advancedSessionAction->setEnabled ( FALSE );
				this->saveSessionAction->setEnabled ( FALSE );
				//				this->advancedSessionAction->setEnabled ( FALSE );
				this->importSessionAction->setEnabled ( TRUE );
			}

			slotStatusMsg ( i18n( "Profile \"%1\" deleted." ).arg( Name ), ID_FLASH_MSG );
			appendLogEntry ( i18n( "Profile \"%1\" deleted." ).arg( Name ) );
			saveOptions();
			this->sessionToggled( "" );
			if ( mw->SessionCombo->currentText().isEmpty() )
				mw->SessionCombo->setEnabled ( FALSE );
		}
	}

}

void KVpnc::sessionToggled( const QString& ) {
	// 	mw->DebugOutput->append( "sessionToggled" );
	if ( mw->SessionCombo->currentText().isEmpty() ) {
		this->deleteSessionAction->setEnabled ( FALSE );
		//			this->advancedSessionAction->setEnabled ( FALSE );
		this->saveSessionAction->setEnabled ( FALSE );
	} else {
		VpnAccountData * it;
		currentProfile = AccountList.first();
		for ( it = AccountList.first(); it; it = AccountList.next() ) {
			QString Name = mw->SessionCombo->currentText();
			if ( !Name.isEmpty() ) {
				if ( it->getName() == Name ) {
					currentProfile = it;

					connectAction->setEnabled( true );

					if ( KvpncDebugLevel > 0 ) {
						QString type;
						if ( currentProfile->getConnectionType() == VpnAccountData::cisco )
							type = "vpnc";
						else if ( currentProfile->getConnectionType() == VpnAccountData::racoon )
							type = "racoon";
						else
							type = "other";

						appendLogEntry( "debug: " + i18n( "New type: %1." ).arg( type ) );
					}
					break;
				}
			}
		}
	}
}

void KVpnc::slotDurationEvent() {
	sec += 1;
	if ( sec > 59 ) {
		sec = 0;
		min += 1;
	}
	if ( min > 59 ) {
		min = 0;
		hour += 1;
	}

	// tmp
	QString hour_string = QString().sprintf( "%2d", hour );
	QString min_string = QString().sprintf( "%2d", min );
	QString sec_string = QString().sprintf( "%2d", sec );

	QString durationString ( i18n( "%1:%2:%3" ).arg( hour_string, min_string, sec_string ) );
	QToolTip::add
		( kickerMenu, i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\"), duration: %4" ).arg( currentProfile->getGateway(), currentProfile->getUserName(), currentProfile->getID(), durationString ) );
	slotStatusMsg( i18n( "Connected: %2@%1, %3" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg ( durationString ), ID_CONNECTION_MSG );
	/*
	runningCheckCounter += 1;
	if ( runningCheckCounter == runningCheckTimeout ) {
	if ( !ConnectProcess->isRunning() ) {
	KMessageBox::error( this, i18n( "Connection lost in last %1 seconds!" ).arg( QString().setNum( runningCheckTimeout ) ) );
	appendLogEntry ( i18n( "Connection to %1 lost at %2" ).arg( mw->gatewayLineEdit->text() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ) );
	terminateConnection ( *ConnectProcess );
	} else {
	runningCheckCounter = 0;
	}
	}
	*/
}

void KVpnc::appendLogEntry( QString msg ) {

	// write to logfile
	if ( writeLogFile ) {
		if ( !logfile.isOpen() ) {
			KStandardDirs * dirs = KGlobal::dirs();
			logfileName = dirs->saveLocation( "data" );
			logfileName += "/kvpnc.log";
			// 			KMessageBox::information( this, logfileName, "info" );
			logfile.setName( logfileName );
			if ( !logfile.open( IO_WriteOnly | IO_Append ) )
				KMessageBox::error( this, i18n( "Log file cant be opened!" ) );
			else {

				s << i18n( "Log session started at: " ) << QDateTime::currentDateTime().toString( Qt::TextDate ) << endl;
			}
		}
		// write into debug console
		mw->DebugOutput->append( msg );
	}
}

void KVpnc::setGuiStatus( int status ) {
	if ( status == connected ) {
		setIcon( connectedIcon );

		// tmp
		QString hour_string = QString().sprintf( "%2d", hour );
		QString min_string = QString().sprintf( "%2d", min );
		QString sec_string = QString().sprintf( "%2d", sec );



		QString durationString ( i18n( "%1:%2:%3" ).arg( hour_string ).arg( min_string ).arg( sec_string ) );
		QToolTip::remove
			( kickerMenu );
		if ( currentProfile->getConnectionType() == VpnAccountData::cisco ) {
			QToolTip::add
				( kickerMenu, i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID:\"%3\"), duration: %4" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg( currentProfile->getID() ).arg ( durationString ) );
			//( kickerMenu, i18n( "Connected to server \"%1\" (user: \"%2\", IPSec ID: \"%3\")" ).arg( mw->gatewayLineEdit->text() ).arg( mw->UsernameLineEdit->text() ).arg( mw->IDLineEdit->text() ) );

			slotStatusMsg ( i18n( "Successful connected." ), ID_FLASH_MSG );
			slotStatusMsg( i18n( "Connected: %2@%1, %3" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg ( durationString ), ID_CONNECTION_MSG );
			appendLogEntry ( i18n( "Successful connected to server \"%1\" (user: \"%2\", IPSec ID: \"%3\") at %4" ).arg( currentProfile->getGateway() ).arg( currentProfile->getUserName() ).arg( currentProfile->getID() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ) );
		}
		if ( currentProfile->getConnectionType() == VpnAccountData::racoon || currentProfile->getConnectionType() == VpnAccountData::freeswan ) {
			QString type = "";
			if ( currentProfile->getConnectionType() == VpnAccountData::racoon )
				type = "racoon";
			if ( currentProfile->getConnectionType() == VpnAccountData::freeswan )
				type = "freeswan";
			slotStatusMsg ( i18n( "Policy successful activated and daemon (%1) running." ).arg( type ), ID_FLASH_MSG );
			appendLogEntry ( i18n( "Policy successful activated and daemon (%3) running for server \"%1\" at %2" ).arg( currentProfile->getGateway() ).arg( QDateTime::currentDateTime().toString( Qt::TextDate ) ).arg( type ) );

		}


		kickerMenu->setStatus( connected );
		mw->buttonConnect->setEnabled( false );
		mw->buttonDisconnect->setEnabled( true );
		this->connectAction->setEnabled( false );
		this->disconnectAction->setEnabled( true );
		if ( minimizeAfterConnect )
			hide();

	} else {
		setIcon( disconnectedIcon );
		mw->buttonConnect->setEnabled( true );
		mw->buttonDisconnect->setEnabled( false );
		this->connectAction->setEnabled( true );
		this->disconnectAction->setEnabled( false );
		kickerMenu->setStatus( disconnected );
		QToolTip::remove
			( kickerMenu );
		QToolTip::add
			( kickerMenu, i18n( "Disconnected" ) );
		slotStatusMsg( i18n( "Disconnected" ), ID_CONNECTION_MSG );
		slotStatusMsg ( i18n( "Disconnected." ), ID_FLASH_MSG );

	}

}

void KVpnc::advancedSettingsClicked() {
	VpnAccountData * it = 0L;
	VpnAccountData *currentAccount = 0L;
	AdvancedSettingsDialog d1;

	d1.AdvancedSettingsCheckbox->setChecked ( false );
	d1.UseGlobalIpsecSecretCheckbox->setChecked ( false );
	d1.SingleDesCheckbox->setChecked( false );
	d1.IkeGroupCheckbox->setChecked ( false );
	d1.LocalPortCheckbox->setEnabled ( false );
	d1.PerfectForwardSecurityCheckbox->setChecked( false );
	d1.NtDomainNameCheckbox->setEnabled ( false );
	d1.ApplicationVersionCheckbox->setEnabled ( false );
	d1.UseGlobalIpsecSecretCheckbox->setEnabled ( false );

	if ( mw->SessionCombo->isEnabled() ) {

		for ( it = AccountList.first(); it; it = AccountList.next() ) {
			if ( it->getName() == mw->SessionCombo->currentText() ) {
				currentAccount = it;

				d1.LocalPortSpinbox->setValue( currentAccount->getLocalPort() );
				d1.IkeGroupCombobox->setCurrentText( currentAccount->getIkeGroup() );
				d1.PerfectForwardSecrecyCombobox->setCurrentText( currentAccount->getPerfectForwardSecrety() );
				d1.NtDomainNameLineedit->setText( currentAccount->getNtDomainName() );
				d1.ApplicationVersionLineedit->setText( currentAccount->getApplicationVersion() );

				if ( currentAccount->getUseAdvancedSettings() ) {
					d1.LocalPortCheckbox->setEnabled ( true );
					d1.IkeGroupCheckbox->setEnabled ( true );
					d1.PerfectForwardSecurityCheckbox->setEnabled ( true );
					d1.SingleDesCheckbox->setEnabled ( true );
					d1.NtDomainNameCheckbox->setEnabled ( true );
					d1.ApplicationVersionCheckbox->setEnabled ( true );
					d1.UseGlobalIpsecSecretCheckbox->setEnabled ( true );
					d1.AdvancedSettingsCheckbox->setChecked ( true );

					if ( currentAccount->getUseSingleDes() ) {
						d1.SingleDesCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseIkeGroup() ) {
						d1.IkeGroupCheckbox->setChecked ( true );
					}

					if ( currentAccount->getUseGlobalIpsecSecret() ) {
						d1.UseGlobalIpsecSecretCheckbox->setChecked ( true );
					}

					if ( currentAccount->getUsePerfectForwardSecrety() ) {
						d1.PerfectForwardSecrecyCombobox->setEnabled ( true );
						d1.PerfectForwardSecurityCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseNtDomainName() ) {
						d1.NtDomainNameLineedit->setEnabled ( true );
						d1.NtDomainNameCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseApplicationVersion() ) {
						d1.ApplicationVersionLineedit->setEnabled ( true );
						d1.ApplicationVersionCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseLocalPort() ) {
						d1.LocalPortSpinbox->setEnabled ( true );
						d1.LocalPortCheckbox->setChecked( true );
					}

					if ( currentAccount->getUseIkeGroup() ) {
						d1.IkeGroupCombobox->setEnabled ( true );
					}

				} // advanced settings

				break;
			} // profile found
		} // for

	}


	d1.exec();

	if ( !mw->SessionCombo->currentText()
	        .isEmpty() ) {
		if ( d1.AdvancedSettingsCheckbox->isChecked() ) {
			currentAccount->setUseAdvancedSettings( true );
		} else {
			currentAccount->setUseAdvancedSettings( false );
		}
		if ( d1.NtDomainNameCheckbox->isChecked() ) {
			currentAccount->setNtDomainName( d1.NtDomainNameLineedit->text() );
			currentAccount->setUseNtDomainName( true );
		} else {
			currentAccount->setUseNtDomainName( false );
		}

		if ( d1.ApplicationVersionCheckbox->isChecked() ) {
			currentAccount->setApplicationVersion( d1.ApplicationVersionLineedit->text() );
			currentAccount->setUseApplicationVersion( true );
		} else {
			currentAccount->setUseApplicationVersion( false );
		}

		if ( d1.IkeGroupCheckbox->isChecked() ) {
			currentAccount->setIkeGroup( d1.IkeGroupCombobox->currentText() );
			currentAccount->setUseIkeGroup( true );
		} else {
			currentAccount->setUseIkeGroup( false );
		}

		if ( d1.PerfectForwardSecurityCheckbox->isChecked() ) {
			currentAccount->setPerfectForwardSecrety( d1.PerfectForwardSecrecyCombobox->currentText() );
			currentAccount->setUsePerfectForwardSecrety( true );
		} else {
			currentAccount->setUsePerfectForwardSecrety( false );
		}


		if ( d1.SingleDesCheckbox->isChecked() ) {
			currentAccount->setUseSingleDes( true );
		} else {
			currentAccount->setUseSingleDes( false );
		}

		if ( d1.UseGlobalIpsecSecretCheckbox->isChecked() ) {
			currentAccount->setUseGlobalIpsecSecret( true );
		} else {
			currentAccount->setUseGlobalIpsecSecret( false );
		}


		if ( d1.LocalPortCheckbox->isChecked() ) {
			currentAccount->setLocalPort( d1.LocalPortSpinbox->value() );
			currentAccount->setUseLocalPort( true );
		} else {
			currentAccount->setUseLocalPort( false );
		}

	} else {
		currentAccount->setUseAdvancedSettings( false );
	}
	saveSessionClicked();

}

void KVpnc::importProfileClicked() {

	ImportProfileDialog dlg;
	dlg.exec();

	bool importOK = dlg.importOk;
	if ( importOK ) {
		VpnAccountData * account = dlg.acc;

		VpnAccountData * it;
		for ( it = AccountList.first(); it; it = AccountList.next() ) {
			if ( it->getName() == account->getName() ) {
				account->setName( QString( account->getName() + "_2" ) );
				KMessageBox::information ( this, i18n( "Profile name exists!\n It will be renamed to \"%1\"." ).arg( account->getName() ), i18n( "Name exist, renamed" ) );

				break;
			}
		}


		AccountList.append( account );
		mw->SessionCombo->insertItem( account->getName() );
		mw->SessionCombo->setCurrentText( account->getName() );

		if ( !mw->SessionCombo->isEnabled() )
			mw->SessionCombo->setEnabled( TRUE );

		sessionToggled( "" );

		KMessageBox::information( 0, i18n( "Import of \"%1\" was successful." ).arg( account->getName() ) );
		slotStatusMsg ( i18n( "Import of \"%1\" successful." ).arg( account->getName() ), ID_FLASH_MSG );
	}

}

void KVpnc::slotCancelClicked() {
	cancelClicked = true;
}

void KVpnc::doRouteRacoon() {

	if ( ConnectProcess->normalExit () ) {
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteRacoon() ) );
		/*
		route add -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/sbin/route" );
		ConnectProcess->addArgument( "add" );
		ConnectProcess->addArgument( "-net" );
		ConnectProcess->addArgument( currentProfile->getRemoteNet() );
		QString dev;

		dev = currentProfile->getNetworkDevice();


		/*	QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
		mw->DebugOutput->append( defaultinterface );
		*/

		ConnectProcess->addArgument( dev );
		tmpInterface = dev;

		if ( currentProfile->getAuthType() != VpnAccountData::psk )
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesRacoon() ) );
		else
			connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doChmodPskFile() ) );

		if ( !ConnectProcess->start() ) {
			KMessageBox::error( this, i18n( "\"route\" start failed!" ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "route: \"route add -net " ) + currentProfile->getRemoteNet() + " " + tmpInterface + "\"" );
		}
	}
}

void KVpnc::doChmodPskFile() {
	disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doChmodPskFile() ) );

	tmpPath = locateLocal ( "data", "kvpnc/" ); // should be
	putenv( ( char* ) ( "LC_ALL=C" ) );
	ConnectProcess->clearArguments();
	ConnectProcess->addArgument( "chmod" );
	ConnectProcess->addArgument( "go-rwx" );
	ConnectProcess->addArgument( tmpPath + "psk." + currentProfile->getName() + ".key" );

	connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesRacoon() ) );

	if ( !ConnectProcess->start() ) {
		KMessageBox::error( this, i18n( "\"chmod go-rwx %1\" start failed!" ).arg( tmpPath + "psk." + currentProfile->getName() + ".key" ) );
		disconnectClicked();
	} else {
		if ( KvpncDebugLevel > 0 )
			mw->DebugOutput->append ( "debug: " + i18n( "\"chmod go-rwx %1\" started." ).arg( tmpPath + "psk." + currentProfile->getName() + ".key" ) );
	}

}

void KVpnc::doIptablesRacoon() {
	if ( ConnectProcess->normalExit () ) {
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesRacoon() ) );
		/*
		iptables -I INPUT -p 50 -i $IFACE -s $VPNGW_IP -j ACCEPT
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/bin/sh" );
		ConnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_racoon.sh" );

		connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRacoon() ) );

		if ( !ConnectProcess->start() ) {
			KMessageBox::error( this, i18n( "\"iptables.%1.add_ipsec_racoon.sh\" start failed!" ).arg( currentProfile->getName() ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				appendLogEntry( "debug: " + i18n( "\"iptables.%1.add_ipsec_racoon.sh\" started" ).arg( currentProfile->getName() ) );
		}
	}
}

void KVpnc::doTailRacoonLog() {

	putenv( ( char* ) ( "LC_ALL=C" ) );
	TailProcess = new QProcess( this );
	TailProcess->addArgument( "tail" );
	TailProcess->addArgument( "-n" );
	TailProcess->addArgument( "100" );
	TailProcess->addArgument( "-f" );
	TailProcess->addArgument( tmpPath + "racoon." + currentProfile->getName() + ".log" );

	connect( TailProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_tail_racoon() ) );
	connect( TailProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_tail_racoon() ) );

	if ( !TailProcess->start() ) {
		KMessageBox::error( this, i18n( "Tail of \"%1\" failed!" ).arg( tmpPath + "racoon." + currentProfile->getName() + ".log" ) );
		//			disconnectClicked();
	} else {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry( "debug: " + i18n( "Tail of \"%1\" started." ).arg( tmpPath + "racoon." + currentProfile->getName() + ".log" ) );
		if ( currentProfile->getDoPingIP() ) {
			sleep (3);
			doPingTest();
		}
	}
}

void KVpnc::doRacoon() {
	if ( ConnectProcess->normalExit () ) {
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRacoon() ) );
		/*
		/usr/sbin/racoon -f <configfile> -l <logfile>
		*/
		QFile f( tmpPath + "racoon." + currentProfile->getName() + ".log");
		if (f.exists())
			f.remove();
		QTextStream s (&f);
			if ( f.open( IO_WriteOnly ) ) {
			s << "\n";
			}
			f.close();
			s.unsetDevice();
		putenv( ( char* ) ( "LC_ALL=C" ) );
		RacoonProcess = new QProcess ( this );
		RacoonProcess->addArgument( pathToRacoon );
		RacoonProcess->addArgument( "-F" ); // foreground we want :)
		RacoonProcess->addArgument( "-f" );
		RacoonProcess->addArgument( tmpPath + "racoon." + currentProfile->getName() + ".conf" );
		RacoonProcess->addArgument( "-l" ); // logfile (we get content with tail process)
		RacoonProcess->addArgument( tmpPath + "racoon." + currentProfile->getName() + ".log" );
		//mw->DebugOutput->append ( "racoon: " +pathToRacoon+ " -f "+tmpPath+"/racoon"+ currentProfile->getName() + ".conf" );

		connect( RacoonProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
		connect( RacoonProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );


		if ( !RacoonProcess->start() ) {
			KMessageBox::error( this, i18n( "\"racoon\" start failed!" ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: racoon: " + pathToRacoon + " -f " + tmpPath + "racoon." + currentProfile->getName() + ".conf" );
			sec = hour = min = 0;

			isConnected = true;
			timer.stop();
			connect( &durationTimer, SIGNAL( timeout() ), this, SLOT( slotDurationEvent() ) );
			durationTimer.start( 1000, FALSE );
			setGuiStatus ( connected );

			doTailRacoonLog();
		}
	}

}

void KVpnc::removeRouteRacoon() {

	if ( DisconnectProcess->normalExit () ) {
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteRacoon() ) );
		/*
		route del -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/sbin/route" );
		DisconnectProcess->addArgument( "del" );
		DisconnectProcess->addArgument( "-net" );
		DisconnectProcess->addArgument( currentProfile->getRemoteNet() );
		QString dev;
		// 		NetworkInterface iface;
		// 		if ( currentProfile->getNetworkDevice() == "default" ) {
		//
		// 			QString defaultinterface=iface.getDefaultInterface();
		// 			dev = defaultinterface;
		// 			DisconnectProcess->addArgument( dev );
		// 		} else
		dev = currentProfile->getNetworkDevice();


		/*	QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
		mw->DebugOutput->append( defaultinterface );
		*/
		if ( KvpncDebugLevel > 0 )
			mw->DebugOutput->append ( "debug: " + i18n( "Network device: %1" ).arg( dev ) );
		DisconnectProcess->addArgument( dev );
		tmpInterface = dev;
		connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesRacoon() ) );

		if ( !DisconnectProcess->start() ) {
			KMessageBox::error( this, i18n( "route del failed!" ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "route: route del -net " ) + currentProfile->getRemoteNet() + " " + tmpInterface );
		}
	}
}

void KVpnc::removeIptablesRacoon() {
	if ( DisconnectProcess->normalExit () ) {
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesRacoon() ) );

		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/bin/sh" );
		DisconnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_racoon.sh" );
		if ( !DisconnectProcess->start() ) {
			KMessageBox::error( this, i18n( "\"iptables.%1.remove_ipsec_racoon.sh\" start failed!" ).arg( currentProfile->getName() ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "\"iptables.%1.remove_ipsec_racoon.sh\" started." ).arg( currentProfile->getName() ) );
		}
	}
}

void KVpnc::removeSetkey() {
	disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeSetkey() ) );

	putenv( ( char* ) ( "LC_ALL=C" ) );
	DisconnectProcess = new QProcess ( this );
	DisconnectProcess->addArgument( "/bin/sh" );
	DisconnectProcess->addArgument( tmpPath + "setkey." + currentProfile->getName() + ".remove_setkey.sh" );

	connect( DisconnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_racoon() ) );
	connect( DisconnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_racoon() ) );
	connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteRacoon() ) );

	if ( !DisconnectProcess->start() ) {
		KMessageBox::error( this, i18n( "setkey (remove) start failed!" ) );
		disconnectClicked();
	} else {
		if ( KvpncDebugLevel > 0 )
			appendLogEntry( "debug: " + i18n( "setkey (remove) started." ) );
	}

	connectionEnded();
}

void KVpnc::doRouteFreeswan() {

	if ( ConnectProcess->normalExit () ) {
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doRouteFreeswan() ) );
		/*
		route add -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/sbin/route" );
		ConnectProcess->addArgument( "add" );
		ConnectProcess->addArgument( "-net" );
		ConnectProcess->addArgument( currentProfile->getRemoteNet() );
		ConnectProcess->addArgument( "ipsec0" );
		QString dev;

		dev = currentProfile->getNetworkDevice();

		NetworkInterface iface;
		QString defaultinterface = "Defaultinterface: " + iface.getDefaultInterface();

		if ( dev.isEmpty() ) {
			if ( defaultinterface.isEmpty() )
				dev = "lo";
			dev = defaultinterface;
		}

		if ( KvpncDebugLevel > 0 )
			mw->DebugOutput->append( "debug: " + i18n( "Default interface: %1" ).arg( defaultinterface ) );

		ConnectProcess->addArgument( dev );
		tmpInterface = dev;
		connect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesFreeswan() ) );

		if ( !ConnectProcess->start() ) {
			KMessageBox::error( this, i18n( "route (freeswan) start failed!" ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "route (freeswan): route add -net " ) + currentProfile->getRemoteNet() + " " + tmpInterface );
		}
	}
}

void KVpnc::doIptablesFreeswan() {
	if ( ConnectProcess->normalExit () ) {
		disconnect( ConnectProcess, SIGNAL( processExited () ), this, SLOT( doIptablesFreeswan() ) );
		/*
		iptables -I INPUT -p 50 -i $IFACE -s $VPNGW_IP -j ACCEPT
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		ConnectProcess->clearArguments();
		ConnectProcess->addArgument( "/bin/sh" );
		ConnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".add_ipsec_freeswan.sh" );

		if ( !ConnectProcess->start() ) {
			KMessageBox::error( this, i18n( "\"iptables.%1.add_ipsec_freeswan.sh\" start failed!" ).arg( currentProfile->getName() ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "\"iptables.%1.add_ipsec_freeswan.sh\" started." ) );
		}
	}
}

void KVpnc::removeRouteFreeswan() {

	if ( DisconnectProcess->normalExit () ) {
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeRouteFreeswan() ) );
		/*
		route del -net 192.168.1.0/24 eth0
		*/
		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/sbin/route" );
		DisconnectProcess->addArgument( "del" );
		DisconnectProcess->addArgument( "-net" );
		DisconnectProcess->addArgument( currentProfile->getRemoteNet() );
		DisconnectProcess->addArgument( "ipsec0" );
		QString dev;
		// 		NetworkInterface iface;
		// 		if ( currentProfile->getNetworkDevice() == "default" ) {
		//
		// 			QString defaultinterface=iface.getDefaultInterface();
		// 			dev = defaultinterface;
		// 			DisconnectProcess->addArgument( dev );
		// 		} else
		dev = currentProfile->getNetworkDevice();


		/*	QString defaultinterface="Defaultinterface: "+iface.getDefaultInterface();
		mw->DebugOutput->append( defaultinterface );
		*/
		mw->DebugOutput->append ( "Device: " + dev );
		DisconnectProcess->addArgument( dev );
		tmpInterface = dev;
		connect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesFreeswan() ) );

		if ( !DisconnectProcess->start() ) {
			KMessageBox::error( this, i18n( "route (freeswan) del failed!" ) );
			disconnectClicked();
		} else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "route(freeswan): route del -net " ) + currentProfile->getRemoteNet() + " " + tmpInterface );
		}
	}
}

void KVpnc::removeIptablesFreeswan() {
	if ( DisconnectProcess->normalExit () ) {
		disconnect( DisconnectProcess, SIGNAL( processExited () ), this, SLOT( removeIptablesFreeswan() ) );

		putenv( ( char* ) ( "LC_ALL=C" ) );
		DisconnectProcess->clearArguments();
		DisconnectProcess->addArgument( "/bin/sh" );
		DisconnectProcess->addArgument( tmpPath + "iptables." + currentProfile->getName() + ".remove_ipsec_freeswan.sh" );
		if ( !DisconnectProcess->start() )
			KMessageBox::error( this, i18n( "\"iptables.%1.remove_ipsec_freeswan.sh\" start failed!" ).arg( currentProfile->getName() ) );
		else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "\"iptables.%1.remove_ipsec_freeswan.sh\" started." ).arg( currentProfile->getName() ) );
		}
	}
	disconnectClicked();
}

void KVpnc::doPingTest() {
	/*
	ping -c3 -I <interfaceIP> <gateway>
	*/
	QString LocalIP;
	QString Interface;
	NetworkInterface iface;
	//LocalIP = iface.getInterfaceIP( iface.getDefaultInterface() );
	if ( currentProfile->getNetworkDevice().isEmpty() )
		Interface = iface.getInterfaceIP( iface.getDefaultInterface() );
	else
		Interface = currentProfile->getNetworkDevice();
	QString PingHostIP = currentProfile->getPingHostIP();
	if ( KvpncDebugLevel > 0 )
		mw->DebugOutput->append( "debug: " + i18n( "Host for ping: %1" ).arg( PingHostIP ) );

	/* first: kick up the tunnel */
	putenv( ( char* ) ( "LC_ALL=C" ) );
	PingProcess = new QProcess( this );
	PingProcess->addArgument( "/bin/ping" );
	PingProcess->addArgument( "-c1" );
	PingProcess->addArgument( PingHostIP );
	if ( !PingProcess->start() )
		KMessageBox::error( this, i18n( "\"ping\" start failed!" ).arg( currentProfile->getName() ) );
	else {
		sleep( 3 );
		PingProcess->kill();
		/* second: real ping */
		PingProcess->clearArguments();
		PingProcess->addArgument( "/bin/ping" );
		PingProcess->addArgument( "-c5" );
		PingProcess->addArgument( "-v" );
		//PingProcess->addArgument( "-I" );
		//PingProcess->addArgument( Interface );
		PingProcess->addArgument( PingHostIP );

		connect( PingProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_ping() ) );
		connect( PingProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_ping() ) );
		connect( PingProcess, SIGNAL( processExited() ), this, SLOT( pingTestExited() ) );

		pingtest = true;

		if ( !PingProcess->start() )
			KMessageBox::error( this, i18n( "\"ping\" start failed!" ).arg( currentProfile->getName() ) );
		else {
			if ( KvpncDebugLevel > 0 )
				mw->DebugOutput->append ( "debug: " + i18n( "\"ping\" started." ) );
				sleep(5);
				PingProcess->kill();
				PingProcess->start();
		}
	}
}

void KVpnc::pingTestExited() {
	disconnect( PingProcess, SIGNAL( processExited () ), this, SLOT( pingTestExited() ) );
	// 	disconnect( PingProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_ping() ) );
	// 	disconnect( PingProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_ping() ) );
	//	pingtest = false;
}

void KVpnc::vpncStarted() {
	disconnect ( ConnectProcess, SIGNAL( processExited () ), this, SLOT( vpncStarted() ) );
	/* doing some after tunnel is up */

	// 		ConnectProcess->clearArguments();
	// 		ConnectProcess->addArgument( "/bin/sh" );
	// 		ConnectProcess->addArgument( VpncConnectScript );
	//
	// 		connect( ConnectProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_vpnc() ) );
	// 		connect( ConnectProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_vpnc() ) );
	//
	// 		if ( !ConnectProcess->start() ) {
	// 			KMessageBox::sorry( this, i18n( "Unable to start process (vpnc-connect)!" ) );
	// 		} else {
	// 			appendLogEntry ( i18n( "vpnc-connect started.\n" ) );
	//
	// 			//                                 connect( &timer, SIGNAL( timeout() ), this, SLOT( slotConnectTimedOut() ) );
	// 			//                                 timer.start( tryConnectTimeout * 1000, FALSE );
	// 			//
	// 			//                                 mw->buttonConnect->setEnabled( false );
	//
	// 		}


}

QString KVpnc::getX509CertificateID( QString Cert, bool user ) {
	issuerID = "";
	userID = "";

	/*
	issuer
	openssl x509 -noout -issuer -in mykeys.pem 

	user
	openssl x509 -noout -subject -in mykeys.pem 
	*/
	putenv( ( char* ) ( "LC_ALL=C" ) );
	X509CertificateExtractProcess = new QProcess ( this );
	X509CertificateExtractProcess->addArgument( pathToOpenssl );
	X509CertificateExtractProcess->addArgument( "x509" );
	X509CertificateExtractProcess->addArgument( "-noout" );
	if ( user )
		X509CertificateExtractProcess->addArgument( "-subject" );
	else
		X509CertificateExtractProcess->addArgument( "-issuer" );

	X509CertificateExtractProcess->addArgument( "-in" );
	X509CertificateExtractProcess->addArgument( Cert );

	X509CertificateExtractUser = user;

	connect( X509CertificateExtractProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_getX509CertificateID() ) );
	connect( X509CertificateExtractProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_getX509CertificateID() ) );
	connect( X509CertificateExtractProcess, SIGNAL( processExited () ), this, SLOT( openSSLEnded() ) );

	if ( !X509CertificateExtractProcess->start() ) {
		KMessageBox::error( this, i18n( "\"openssl\" (getCertificate) start failed!" ) );
		return "";
	} else {
		//KMessageBox::information( this, i18n( "openssl started!" ) );
		if ( KvpncDebugLevel > 0 )
			mw->DebugOutput->append( "debug: " + i18n( "\"openssl\" (getCertificate) started." ) );
		X509CertificateExtractRunning = true;

		while ( X509CertificateExtractRunning ) {
			if ( KvpncDebugLevel > 0 )
				appendLogEntry( "debug: " + i18n( "Waiting for process end (getCertificate)." ) );
			parent->QApplication::processEvents();
		}
		disconnect( X509CertificateExtractProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_getX509CertificateID() ) );
		disconnect( X509CertificateExtractProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_getX509CertificateID() ) );
		disconnect( X509CertificateExtractProcess, SIGNAL( processExited () ), this, SLOT( openSSLEnded() ) );
		delete X509CertificateExtractProcess;
		if ( user )
			return userID;
		else
			return issuerID;
	}

}

void KVpnc::readFromStdout_getX509CertificateID() {
	QString ProcessMsg = "";
	if ( X509CertificateExtractProcess->canReadLineStdout() )
		ProcessMsg = X509CertificateExtractProcess->readLineStdout();
	if ( !ProcessMsg.isEmpty() ) {
		if ( KvpncDebugLevel > 0 )
			mw->DebugOutput->append( "debug: " + i18n( "ID found: %1" ).arg( ProcessMsg.section( '=', 1 ) ) );
		if ( X509CertificateExtractUser ) {
			userID = ProcessMsg.section( '=', 1 ); // subject = /C=DE... => /C=DE...
			if ( userID.left( 0 ) == ' ' )
				userID = userID.right( userID.length() - 1 );

		} else {
			issuerID = ProcessMsg.section( '=', 1 ); // issuer= /C=DE... => /C=DE...
			if ( issuerID.left( 0 ) == ' ' )
				issuerID = issuerID.right( issuerID.length() - 1 );
		}
	}
}

void KVpnc::readFromStderr_getX509CertificateID() {
	QString ProcessMsg = QString( X509CertificateExtractProcess->readStderr() );
	if ( KvpncDebugLevel > 0 )
		mw->DebugOutput->append( "debug: " + i18n( "getX509CertificateID() err: %1" ).arg( ProcessMsg ) );
}

void KVpnc::openSSLEnded() {
	X509CertificateExtractRunning = false;
	if ( KvpncDebugLevel > 0 )
		mw->DebugOutput->append( "debug: " + i18n( "OpenSSL finished.\n" ) );

}

void KVpnc::importCertClicked() {
	ImportP12CertificateDialog dlg( mw->DebugOutput );
	//int result =
	dlg.exec();
}

void KVpnc::showProfileManagerClicked() {
	ProfileManager manager( AccountList, mw->DebugOutput, KvpncDebugLevel, currentProfile, this, "ProfileManger" );
	manager.exec();

}

#include "kvpnc.moc"
