/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   Contains parts of code from klcc project                              *
*   (Peter Simonsson <psn@linux.se>).                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef __KVPNC_H__
#define __KVPNC_H__

#include <qwidget.h>
#include <qstrlist.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qptrlist.h>
#include <qtextstream.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qsize.h>
#include <kapp.h>
#include <kmainwindow.h>
#include <kaction.h>
#include <kconfig.h>
#include <kprocess.h>
#include <kwizard.h>
#include <kapp.h>
#include "kvpnckicker.h"
#include "preferencesdialog.h"
#include "mainview.h"
#include "vpnaccountdata.h"
#include "networkinterface.h"

// ID for statusbar
#define ID_STATUS_MSG 1
#define ID_CONNECTION_MSG 2
#define ID_FLASH_MSG 3

/**
* main class of kvpnc
* @author Christoph Thielecke
*/
class KVpnc : public KMainWindow {
		Q_OBJECT
	public:
		/**
		* Konstruktor
		* @param parent Parent widget
		* @param name Name of the Mainwindow
		*/
		KVpnc( KApplication *parent, QWidget*parent = 0, const char *name = 0 );

		/**
		* Destruktor
		*/
		~KVpnc();
	protected:
		/**
		* init the actions
		*/
		void initAction();

		/**
		* init the main view
		*/
		void initView();

		/**
		* init the statusbar
		*/
		void initStatusbar();

		/**
		* init the kicker system tray icon
		*/
		void initDockWindow();

		/**
		* save configuration
		*/
		void saveOptions();

		/**
		* load configuration
		*/
		void loadOptions();

		/**
		* sets the gui status
		* @param status The status that will be set
		*/
		void setGuiStatus( int status );

		/**
		* terminates the connection
		* @param proc the connect process
		*/
		void terminateConnection ( QProcess& proc );
		/**
		* called when quit requested
		*/
		virtual bool queryExit();

		/**
		* return the user id of the certificate
		*	@param Cert certificate from wich should be extracted
		* @param user true if user id, false if issuer id
		*/
		QString getX509CertificateID( QString Cert, bool user );


	private:
		KVpncKicker* kickerMenu; //< kicker menu
		KAction* connectAction; //< connect action
		KAction* disconnectAction; //< disconnect action
		KAction *newSessionAction;
		//	KAction *advancedSessionAction;
		KAction *saveSessionAction;
		KAction *deleteSessionAction;
		KAction *importSessionAction;
		KAction *importCertAction;
		KAction* selectAction; //< line select action
		KAction* profileManagerAction;
		KAction* helpAction;
		KToggleAction* showToolbarAction; //< toggle toolbar on/off
		KToggleAction* showStatusbarAction; //< toggle statusbar on/off
		KAction* preferencesAction; //< show preferences dialog
		KConfig* config; //< configuration data
		MainView* mw; //< Main gui
		KApplication *parent;
		VpnAccountData *currentProfile;
		QPtrList<VpnAccountData> AccountList;
		QTimer timer;
		QTimer durationTimer; //< timer for counting connection time
		int connectCounter; //< counter for connecting
		int connectSleepDelay; //< time beetween retry
		int connectSleepRetry; //< count of retries
		int tryConnectTimeout; //< timeout in at connecting
		int VpncDebugLevel;
		int RacoonDebugLevel;
		int KvpncDebugLevel;
		int connectionStartTime;
		int runningCheckCounter;
		int runningCheckTimeout;
		int sec, min, hour;
		QPoint WindowPos;
		QSize WindowSize;
		bool minimizeAfterConnect;
		bool showDebugConsole;
		bool isConnected;
		bool writeLogFile;
		bool autoclearFields;
		bool cancelClicked;
		bool deviceaddrstr2addr;
		bool pingtest;
		bool useDefaultPaths;
		bool getUserID;
		bool X509CertificateExtractRunning;
		bool X509CertificateExtractUser;
		bool useSilentQuit;
		bool holdGeneratedScripts;
		QString Vpnc_pid_file;
		QString OldDefaultroute;
		QString VpncConnectScript;
		QString VpncDisconnectScript;
		QString pathToVpnc;
		QString pathToRacoon;
		QString pathToSetkey;
		QString pathToIptables;
		QString pathToOpenssl;
		QString pathToFreeswan;
		QString tmpPath;
		QString ProcessMsg_connect;
		QString ProcessMsg_disconnect;
		QString lastProfile;
		QString logfileName;
		QString deviceaddrstr;
		QString deviceaddr;
		QString tmpInterface;
		QString VpncGateway;
		QString VpncScript;
		QString issuerID; // for id extract
		QString userID;   // for id extract
		QProcess *ConnectProcess;
		QProcess *DisconnectProcess;
		QProcess *RacoonProcess;
		QProcess *FreeswanProcess;
		QProcess *PingProcess;
		QProcess *TailProcess;
		QProcess *X509CertificateExtractProcess;
		QTime connectionTime;
		QFile logfile;
		QTextStream s;
		QPixmap connectedIcon;
		QPixmap disconnectedIcon;
		QPoint pos;

		


		



		enum status {disconnected, connected};

	public slots:
		/** setup KVpnc */
		void slotSettings();
		/** show / hide toolbar */
		void slotShowToolbar();
		/** show / hide statusbar */
		void slotShowStatusbar();
		/** configure key bindings */
		void slotKeyBindings();
		/** configure toolbars */
		void slotConfigToolbars();

		void slotConfigNotifications();

		/**
		* Append a log entry
		* @param msg The string that will be appended
		*/
		void appendLogEntry( QString msg );

		/**
		* Slot for status message
		* @param text The text that will be set
		* @param id What type its is
		*/
		void slotStatusMsg( const QString &text, int id = ID_STATUS_MSG );


	public slots:
		void connectClicked();
		void disconnectClicked();
		void saveSessionClicked();
		void newSessionClicked();
		void deleteSessionClicked();
		void sessionToggled( const QString& );
		void helpClicked();

	private slots:
		void closeEvent( QCloseEvent* e );
		void slotCancelClicked();
		void slotConnectTimedOut();
		void slotDurationEvent();
		void readFromStdout_vpnc();
		void readFromStderr_vpnc();
		void readFromStdout_disconnect();
		void readFromStderr_disconnect();
		void readFromStdout_racoon();
		void readFromStderr_racoon();
		void readFromStdout_tail_racoon();
		void readFromStderr_tail_racoon();
		void readFromStdout_ping();
		void readFromStderr_ping();
		void readFromStdout_getX509CertificateID();
		void readFromStderr_getX509CertificateID();
		void quitCalled();
		void advancedSettingsClicked();
		void importProfileClicked();
		void doRacoon();
		void doRouteRacoon();
		void doRouteFreeswan();
		void doChmodPskFile();
		void doTailRacoonLog();
		void doIptablesRacoon();
		void doIptablesFreeswan();
		void removeRouteRacoon();
		void removeRouteFreeswan();
		void removeIptablesRacoon();
		void removeIptablesFreeswan();
		void removeSetkey();
		void connectionEnded();
		void doPingTest();
		void pingTestExited();
		void vpncStarted();
		void openSSLEnded();
		void importCertClicked();
		void showProfileManagerClicked();
};

#endif

