/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "profilewidget.h"
#include "importp12certificatedialog.h"
#include <qlistview.h>
#include <klistview.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <qlistbox.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <knuminput.h>
#include "networkinterface.h"
#include "advancedsettingsdialog.h"

ProfileWidget::ProfileWidget( QPtrList<VpnAccountData>& AccountList, QTextEdit *LogOutput, int KvpncDebugLevel, VpnAccountData *currentProfile ) {
	this->AccountList = AccountList;
	this->LogOutput = LogOutput;
	this->currentProfile = currentProfile;
	this->KvpncDebugLevel = KvpncDebugLevel;

	// TODO read types from list
	ConnectionTypeComboBox->insertItem( i18n( "Cisco (vpnc)" ), 0 );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (racoon)" ), 1 );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (FreeSWAN)" ), 2 );
	ConnectionTypeComboBox->insertItem( "-", 3 );

	// profile data
	VpnAccountData * it;
	for ( it = AccountList.first(); it; it = AccountList.next() ) {
		//KMessageBox::information(0,"item found",it->getName());

		ProfileListBox->insertItem( it->getName() );
	}

	connect( ProfileListBox, SIGNAL( highlighted ( QListBoxItem * ) ), this, SLOT( profileChanged( QListBoxItem * ) ) );
	connect( ConnectionTypeComboBox, SIGNAL( activated( const QString& ) ), this, SLOT( connectionTypeChanged( const QString& ) ) );
	connect( AuthTypeComboBox , SIGNAL( activated( const QString& ) ), this, SLOT( authTypeChanged( const QString& ) ) );
	connect( AdvancedSettingsPushButton, SIGNAL( clicked () ), this, SLOT( advancedSettingClicked() ) );
	connect( PingCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( pingIpToggled( bool ) ) );

	if ( !AccountList.isEmpty() ) {
		if ( currentProfile != 0 )
			ProfileListBox->setCurrentItem( ProfileListBox->findItem( currentProfile->getName() ) );
		else
			ProfileListBox->setCurrentItem( 0 );
	}

	NetworkInterface iface;
	QStringList list = iface.getAllNetworkInterfaces();
	NetworkDeviceComboBox->insertItem( "default" );
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
		NetworkDeviceComboBox->insertItem( QString( *it ) );
	}
	NetworkDeviceComboBox->setEnabled( true );

	if ( currentProfile != 0 ) {
		if ( currentProfile->getAuthType() == VpnAccountData::cert )
			AuthTypeComboBox->setCurrentText( i18n( "X.509 certificate" ) );

		if ( currentProfile->getAuthType() == VpnAccountData::psk )
			AuthTypeComboBox->setCurrentText( i18n( "Pre shared key" ) );

		PSKLineEdit->setText( currentProfile->getPreSharedKey() );

		if ( currentProfile->getDoPingIP() )
			PingIPLineEdit->setEnabled( true );
		else
			PingIPLineEdit->setEnabled( false );
	}

	RemoteIDLineEdit->setEnabled( false );

	connectionTypeChanged( ConnectionTypeComboBox->currentText() );
	authTypeChanged( AuthTypeComboBox->currentText() );
	profileChanged( ProfileListBox->item( ProfileListBox->currentItem() ) );

}


ProfileWidget::~ProfileWidget() {}


void ProfileWidget::importCertificate() {
	ImportP12CertificateDialog dlg( LogOutput );
	//int result =
	dlg.exec();
}

void ProfileWidget::profileChanged( QListBoxItem * ) {

	//int item = ProfileListBox->currentItem();
	QString Name = ProfileListBox->currentText();
	if ( KvpncDebugLevel > 0 )
		LogOutput->append( i18n( "new profile: %1" ).arg( Name ) );

	//	KMessageBox::information(0,"item clicked",Name);
	// profile data
	VpnAccountData * it;
	for ( it = AccountList.first(); it; it = AccountList.next() ) {
		if ( it->getName() == Name ) {
			currentProfile = it;
			break;
		}
	}

	x509certURLRequester->clear();
	certpathURLRequester->clear();
	UsernameLineEdit->clear();
	GroupPasswordEdit->clear();
	PSKLineEdit->clear();

	CheckGroupPass->setChecked( false );
	CheckUserPass->setChecked( false );
	IDLineEdit->clear();
	UsernameLineEdit->clear();
	UsernameLineEdit->setEnabled( false );
	PasswordEdit->clear();
	PasswordEdit->setEnabled( false );
        if ( currentProfile == 0 )
            return;

	// 	x509certURLRequester->setEnabled( false );
	// 	certpathURLRequester->setEnabled( false );
	// 	AdvancedSettingsPushButton->setEnabled( false );
	if ( ( currentProfile->getNetworkDevice() ).isEmpty() ) {
		NetworkDeviceComboBox->setCurrentText( "default" );
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "profile \"%1\": no network device definied, using \"default\"." ).arg( currentProfile->getName() ) );
	} else {
		NetworkDeviceComboBox->setCurrentText( currentProfile->getNetworkDevice() );
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "profile \"%1\": network device definied, using \"%2\"." ).arg( currentProfile->getName() ).arg( currentProfile->getNetworkDevice() ) );
	}

	// global
	if ( it->getConnectionType() == VpnAccountData::cisco )
		ConnectionTypeComboBox->setCurrentItem( 0 );
	else if ( it->getConnectionType() == VpnAccountData::racoon )
		ConnectionTypeComboBox->setCurrentItem( 1 );
	else if ( it->getConnectionType() == VpnAccountData::freeswan )
		ConnectionTypeComboBox->setCurrentItem( 2 );
	else
		ConnectionTypeComboBox->setCurrentItem( 3 ); // must be last item -> other

	gatewayLineEdit->setText( it->getGateway() );

	PSKLineEdit->setText( currentProfile->getPreSharedKey() );

	if ( it->getConnectionType() == VpnAccountData::cisco ) {
		CheckGroupPass->setChecked( it->getSaveGroupPassword() );
		CheckUserPass->setChecked ( it->getSaveUserPassword() );
		IDLineEdit->setText( it->getID() );
		UsernameLineEdit->setText ( it->getUserName() );
		GroupPasswordEdit->setText( it->getGroupPassword() );
		PasswordEdit->setText ( it->getUserPassword() );
		PSKLineEdit->setText( it->getPreSharedKey() );

		CheckGroupPass->setEnabled( true );
		CheckUserPass->setEnabled( true );
		IDLineEdit->setEnabled( true );
		UsernameLineEdit->setEnabled( true );
		GroupPasswordEdit->setEnabled( true );
		AdvancedSettingsPushButton->setEnabled( true );
		PasswordEdit->setEnabled( true );
		GroupPasswordEdit->setEnabled( true );

		x509certURLRequester->setEnabled( false );
		certpathURLRequester->setEnabled( false );
		AuthTypeComboBox->setEnabled( false );
		PSKLineEdit->setEnabled( false );
		RemoteNetAddrLineEdit->setEnabled( false );
		RemoteNetMaskSpinBox->setEnabled( false );
		RemoteIDLineEdit->setEnabled( false );
		UseSpecialRemoteIDcheckBox->setEnabled( false );
	} // vpnc

	else if ( it->getConnectionType() == VpnAccountData::racoon || it->getConnectionType() == VpnAccountData::freeswan ) {
		//     KURLRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
		int RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 ).toInt();
		RemoteNetAddrLineEdit->setText( RemoteNetAddr );
		RemoteNetMaskSpinBox->setValue( RemoteNetMask );
		if ( it->getAuthType() == VpnAccountData::cert )
			AuthTypeComboBox->setCurrentItem( 0 );
		else
			AuthTypeComboBox->setCurrentItem( 1 );


		x509certURLRequester->setURL( currentProfile->getX509Certificate() );
		certpathURLRequester->setURL( currentProfile->getCertPath() );
		PingCheckBox->setChecked( currentProfile->getDoPingIP() );
		PingIPLineEdit->setText( currentProfile->getPingHostIP() );
		RemoteIDLineEdit->setText( currentProfile->getSpecialRemoteID() );
		UseSpecialRemoteIDcheckBox->setChecked( currentProfile->getUseSpecialRemoteID() );

		x509certURLRequester->setEnabled( true );
		certpathURLRequester->setEnabled ( true );
		AuthTypeComboBox->setEnabled( true );
		PSKLineEdit->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( true );
		RemoteNetMaskSpinBox->setEnabled( true );

		UseSpecialRemoteIDcheckBox->setEnabled( true );
		ImportCertificatePushButton->setEnabled( true );

		CheckGroupPass->setEnabled( false );
		CheckUserPass->setEnabled( false );
		IDLineEdit->setEnabled( false );
		UsernameLineEdit->setEnabled( false );
		GroupPasswordEdit->setEnabled( false );
		AdvancedSettingsPushButton->setEnabled( false );
		PasswordEdit->setEnabled( false );
		GroupPasswordEdit->setEnabled( false );

	} // racoon or freeswan

	connectionTypeChanged( ConnectionTypeComboBox->currentText() );
	authTypeChanged( AuthTypeComboBox->currentText() );
	x509certURLRequester->setFilter( "*.pem" );
	certpathURLRequester->setMode( KFile::Directory );
}

void ProfileWidget::connectionTypeChanged( const QString& ) {

	DeleteSessionPushButton->setEnabled ( TRUE );
	NewSessionPushButton->setEnabled ( TRUE );
	SaveSessionPushButton->setEnabled ( TRUE );

	// First we should set a type
	// 	parent->buttonCancel->setEnabled( FALSE );
	// 	parent->buttonOk->setEnabled( FALSE );
	AdvancedSettingsPushButton->setEnabled( false );

	int item = ConnectionTypeComboBox->currentItem();

	if ( item == VpnAccountData::ConnectionType( VpnAccountData::cisco ) ) {
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( "new type: cisco" );

		CheckGroupPass->setEnabled( true );
		CheckUserPass->setEnabled( true );
		IDLineEdit->setEnabled( true );
		UsernameLineEdit->setEnabled( true );
		GroupPasswordEdit->setEnabled( true );
		PasswordEdit->setEnabled( true );
		AdvancedSettingsPushButton->setEnabled( true );

		// 		x509certURLRequester->setEnabled( false );
		// 		certpathURLRequester->setEnabled( false );
		NetworkDeviceComboBox->setEnabled( true );
		AuthTypeComboBox->setEnabled( false );
		// 		PSKLineEdit->setEnabled( false );
		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( false );
		RemoteNetMaskSpinBox->setEnabled( false );
		privkeypathURLRequester->setEnabled( false );
		PrivkeyPasswordEdit->setEnabled( false );
		ImportCertificatePushButton->setEnabled( false );
	} // vpnc

	else if ( item == VpnAccountData::racoon || item == VpnAccountData::freeswan ) {

		if ( item == VpnAccountData::racoon ) {
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "new type: racoon" ) );
			privkeypathURLRequester->setEnabled( false );
			PrivkeyPasswordEdit->setEnabled( false );
		} else {
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "new type: freeswan" ) );
			privkeypathURLRequester->setEnabled( true );
			PrivkeyPasswordEdit->setEnabled( true );
			privkeypathURLRequester->setURL( currentProfile->getPrivateKey() );
			PrivkeyPasswordEdit->setText( currentProfile->getPrivateKeyPass() );
		}
		//     KURLRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		// 		x509certURLRequester->setEnabled( true );
		// 		certpathURLRequester->setEnabled( true );
		//ImportCertificatePushButton->setEnabled( true );
		NetworkDeviceComboBox->setEnabled ( true );
		AuthTypeComboBox->setEnabled( true );
		// 		PSKLineEdit->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( true );
		RemoteNetMaskSpinBox->setEnabled( true );
		PasswordEdit->setEnabled( false );
		CheckGroupPass->setEnabled( false );
		CheckUserPass->setEnabled( false );
		IDLineEdit->setEnabled( false );
		GroupPasswordEdit->setEnabled( false );
		PasswordEdit->setEnabled( false );
		UsernameLineEdit->setEnabled( false );
		AdvancedSettingsPushButton->setEnabled( false );

		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );
		if ( UseSpecialRemoteIDcheckBox->isChecked() )
			RemoteIDLineEdit->setEnabled( true );
		else
			RemoteIDLineEdit->setEnabled( false );
	} // racoon/freeswan

	// other
	else if ( item == VpnAccountData::other ) {
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "new type: none specified" ) );
		// 		x509certURLRequester->setEnabled( false );
		// 		AuthTypeComboBox->setEnabled( false );
		// 		PSKLineEdit->setEnabled( false );
		//
		// 		CheckGroupPass->setEnabled( false );
		// 		CheckUserPass->setEnabled( false );
		// 		IDLineEdit->setEnabled( false );
		// 		UsernameLineEdit->setEnabled( false );
		// 		GroupPasswordEdit->setEnabled( false );
		// 		AdvancedSettingsPushButton->setEnabled( false );
		// 		buttonCancel->setEnabled( FALSE );
		// 		buttonOk->setEnabled( FALSE );
	} else {
		LogOutput->append( i18n( "unknown type" ) );
	}
	authTypeChanged( "" );
}

void ProfileWidget::authTypeChanged( const QString& ) {

	// racoon or freeswan
	if ( ConnectionTypeComboBox->currentItem() == 1 || ConnectionTypeComboBox->currentItem() == 2 ) {

		// cert
		if ( AuthTypeComboBox->currentItem() == 0 ) {
			// certificate
			PSKLineEdit->setEnabled( false );
			x509certURLRequester->setEnabled( true );
			certpathURLRequester->setEnabled( true );
			ImportCertificatePushButton->setEnabled( true );
		}

		// psk
		if ( AuthTypeComboBox->currentItem() == 1 ) {
			// psk
			PSKLineEdit->setEnabled( true );
			x509certURLRequester->setEnabled( false );
			certpathURLRequester->setEnabled( false );
			ImportCertificatePushButton->setEnabled( false );
		}
	}
}

void ProfileWidget::newSessionClicked() {

	QString Name = "";
	bool nameOk = true;
	cancelClicked = false;
	KLineEditDlg dlg( i18n( "Enter name for new Profile:" ), NULL, 0 );
	connect( &dlg, SIGNAL( cancelClicked() ), this, SLOT( slotCancelClicked() ) );
	while ( ( !( !Name.isEmpty() && nameOk ) ) || cancelClicked ) {
		nameOk = true;

		dlg.exec();
		if ( !cancelClicked ) {
			Name = dlg.text();

			VpnAccountData * it;
			for ( it = AccountList.first(); it; it = AccountList.next() ) {
				if ( it->getName() == Name ) {
					KMessageBox::error ( this, i18n( "Profile name exists!" ), i18n( "Name exist" ) );
					nameOk = false;
					break;
				}
				if ( Name.find( " " ) > -1 || Name.isEmpty() ) {
					KMessageBox::error ( this, i18n( "Profile name can't contain spaces!" ), i18n( "Illegal character" ) );
					nameOk = false;
					break;
				}
			}
		} else
			break;
	}
	if ( !Name.isEmpty() && !cancelClicked ) {

		x509certURLRequester->clear();
		certpathURLRequester->clear();
		UsernameLineEdit->clear();
		GroupPasswordEdit->clear();
		PSKLineEdit->clear();

		CheckGroupPass->setChecked( false );
		CheckUserPass->setChecked( false );
		IDLineEdit->clear();
		UsernameLineEdit->clear();
		GroupPasswordEdit->clear();

		DeleteSessionPushButton->setEnabled ( TRUE );
		NewSessionPushButton->setEnabled ( TRUE );
		SaveSessionPushButton->setEnabled ( TRUE );

		// First we should set a type
		// 		buttonCancel->setEnabled( FALSE );
		// 		buttonOk->setEnabled( FALSE );
		AdvancedSettingsPushButton->setEnabled( false );

		// currently static for new profile
		VpnAccountData::ConnectionType ConnectionType = VpnAccountData::other;
		QString Gateway = "";
		QString ID = "";
		QString Username = "";
		QString UserPassword = "";
		QString GroupPassword = "";
		bool saveUserPassword = false;
		bool saveGroupPassword = false;

		QString ApplicationVersion = "";
		int Port = 500;
		QString NtDomainName = "";
		QString PerfectForwardSecrecy = "";
		QString IkeGroup = "";
		bool useApplicationVersion = false;
		bool useGlobalIpsecSecret = false;
		bool useIkeGroup = false;
		bool useLocalPort = false;
		bool useNtDomainName = false;
		bool useSingleDes = false;
		bool useAdvancedSettings = false;
		bool usePerfectSecrecy = false;

		AccountList.append( new VpnAccountData( ConnectionType, Name, Gateway, ID, GroupPassword, Username, UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings, usePerfectSecrecy ) );

		ProfileListBox->insertItem( Name, 0 );
		ProfileListBox->setCurrentItem ( 0 );
		ConnectionTypeComboBox->setCurrentItem( 3 ); // other

		KMessageBox::information ( this, i18n( "Profile \"%1\" added.\nNow set the right type,\n fill in the fields\nand press \"Save\" to complete the profile." ).arg( Name ), i18n( "Complete profile" ) );
		//slotStatusMsg ( i18n( "Profile \"%1\" added." ).arg( Name ), ID_FLASH_MSG );
		LogOutput->append ( i18n( "Profile \"%1\" added." ).arg( Name ) );
	}
}


void ProfileWidget::saveSessionClicked() {
    if ( currentProfile== 0 )
        return;
	//QString itemString=ConnectionTypeComboBox->currentText();
	int item = ConnectionTypeComboBox->currentItem();
	// cisco
	if ( item == VpnAccountData::cisco )
		currentProfile->setConnectionType( VpnAccountData::cisco );
	else if ( item == VpnAccountData::racoon || item == VpnAccountData::freeswan ) {
		currentProfile->setSpecialRemoteID( RemoteIDLineEdit->text() );
		currentProfile->setUseSpecialRemoteID( UseSpecialRemoteIDcheckBox->isChecked() );

		if ( item == VpnAccountData::racoon )
			currentProfile->setConnectionType( VpnAccountData::racoon );
		else {
			currentProfile->setConnectionType( VpnAccountData::freeswan );
			currentProfile->setPrivateKey( privkeypathURLRequester->url() );
			currentProfile->setPrivateKeyPass( PrivkeyPasswordEdit->text() );
		}

		bool validAddr = true;
		if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 ) {
			KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
			validAddr = false;
		} else {
			QString addr = RemoteNetAddrLineEdit->text();
			int part0 = addr.section( '.', 0, 0 ).toInt();
			int part1 = addr.section( '.', 1, 1 ).toInt();
			int part2 = addr.section( '.', 2, 2 ).toInt();
			int part3 = addr.section( '.', 3, 3 ).toInt();

			/*
						LogOutput->append ("part0: "+QString().setNum(part0));
						LogOutput->append ("part1: "+QString().setNum(part1));
						LogOutput->append ("part2: "+QString().setNum(part2));
						LogOutput->append ("part3: "+QString().setNum(part3));
			*/
			if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 1 || part1 > 254 ) || ( part2 < 1 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) ) {
				KMessageBox::error ( this, i18n( "IP address (remote net) is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = false;
			}
			if ( validAddr == true ) {
				currentProfile->setRemoteNet( addr + "/" + QString().setNum( RemoteNetMaskSpinBox->value() ) );
			}

		}


		if ( AuthTypeComboBox->currentItem() == 0 )
			currentProfile->setAuthType( VpnAccountData::cert );
		else
			currentProfile->setAuthType( VpnAccountData::psk );

		currentProfile->setX509Certificate( x509certURLRequester->url() );
		currentProfile->setCertPath( certpathURLRequester->url() );

		currentProfile->setPreSharedKey( PSKLineEdit->text());


	} else
		currentProfile->setConnectionType( VpnAccountData::other );

	if ( PingIPLineEdit->text() != QString( "" ) ) {
		bool validAddr = true;
		if ( PingIPLineEdit->text().contains( '.' ) != 3 ) {
			KMessageBox::error ( this, i18n( "No IP address (ping host) entered!" ), i18n( "No IP address" ) );
			validAddr = false;
		} else {
			QString addr = PingIPLineEdit->text();
			int part0 = addr.section( '.', 0, 0 ).toInt();
			int part1 = addr.section( '.', 1, 1 ).toInt();
			int part2 = addr.section( '.', 2, 2 ).toInt();
			int part3 = addr.section( '.', 3, 3 ).toInt();

			if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 1 || part1 > 254 ) || ( part2 < 1 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) ) {
				KMessageBox::error ( this, i18n( "IP address (ping host) is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = false;
			}
			if ( validAddr == true ) {
				currentProfile->setPingHostIP( addr );
			}
		}
	} else {
		if ( PingCheckBox->isChecked() && PingIPLineEdit->text() == QString( "" ) )
			KMessageBox::error ( this, i18n( "No IP address (ping host) entered!" ), i18n( "No IP address" ) );
		else
			currentProfile->setPingHostIP( "" );
	}


	currentProfile->setSaveGroupPassword( CheckGroupPass->isChecked() );
	currentProfile->setSaveUserPassword( CheckUserPass->isChecked () );
	currentProfile->setID( IDLineEdit->text() );
	currentProfile->setUserName( UsernameLineEdit->text () );
	currentProfile->setGroupPassword( GroupPasswordEdit->text( ) );
	currentProfile->setUserPassword( PasswordEdit->text ( ) );
	currentProfile->setGateway( gatewayLineEdit->text() );
	currentProfile->setPreSharedKey( PSKLineEdit->text() );
	currentProfile->setX509Certificate( x509certURLRequester->url() );
	currentProfile->setCertPath( certpathURLRequester->url() );
	currentProfile->setNetworkDevice( NetworkDeviceComboBox->currentText() );

	currentProfile->setDoPingIP( PingCheckBox->isChecked() );

	if ( AuthTypeComboBox->currentItem() == 0 )
		currentProfile->setAuthType( VpnAccountData::cert );
	if ( AuthTypeComboBox->currentItem() == 1 )
		currentProfile->setAuthType( VpnAccountData::psk );

	LogOutput->append( i18n( "Profile %1 saved." ).arg( currentProfile->getName() ) );
}


void ProfileWidget::deleteSessionClicked() {
	QString Name = ProfileListBox->currentText();
	if ( !ProfileListBox->currentText().isEmpty() ) {
		int result = KMessageBox::questionYesNo ( this, i18n( "Really delete \"%1\"?" ).arg( Name ), i18n( "Delete?" ) );
		if ( result ) {
			ProfileListBox->removeItem( ProfileListBox->currentItem() );

			VpnAccountData * it;
			for ( it = AccountList.first(); it; it = AccountList.next() ) {
				if ( ProfileListBox->currentText().isEmpty() ) {
					// 					this->deleteSessionAction->setEnabled ( FALSE );
					//			this->advancedSessionAction->setEnabled ( FALSE );
					// 					this->saveSessionAction->setEnabled ( FALSE );
				} else {
					if ( it->getName() == Name ) {
						AccountList.remove( it );
						break;
					}
				}
			}


			if ( !AccountList.isEmpty() ) {
				AccountList.first();
				ProfileListBox->setCurrentItem( 0 );
				// 				buttonCancel->setEnabled( true );
				// 				buttonOk->setEnabled( true );
			} else {
				// 				this->deleteSessionAction->setEnabled ( FALSE );
				//				this->advancedSessionAction->setEnabled ( FALSE );
				// 				this->saveSessionAction->setEnabled ( FALSE );
				//				this->advancedSessionAction->setEnabled ( FALSE );
				// 				this->importSessionAction->setEnabled ( TRUE );
			}

			//		slotStatusMsg ( i18n( "Profile \"%1\" deleted." ).arg( Name ), ID_FLASH_MSG );
			LogOutput->append ( i18n( "Profile \"%1\" deleted." ).arg( Name ) );
			//saveOptions(); // TODO save, currently after closing dialog
			//			sessionToggled();
			//if ( ProfileListBox->currentText().isEmpty() )
			//	ProfileListBox->setEnabled ( FALSE );
		}
	}

}

void ProfileWidget::slotCancelClicked() {
	cancelClicked = true;
}

void ProfileWidget::advancedSettingClicked() {
	AdvancedSettingsDialog d1;

	d1.AdvancedSettingsCheckbox->setChecked ( false );
	d1.UseGlobalIpsecSecretCheckbox->setChecked ( false );
	d1.SingleDesCheckbox->setChecked( false );
	d1.IkeGroupCheckbox->setChecked ( false );
	d1.LocalPortCheckbox->setEnabled ( false );
	d1.PerfectForwardSecurityCheckbox->setChecked( false );
	d1.NtDomainNameCheckbox->setEnabled ( false );
	d1.ApplicationVersionCheckbox->setEnabled ( false );
	d1.UseGlobalIpsecSecretCheckbox->setEnabled ( false );

	d1.LocalPortSpinbox->setValue( currentProfile->getLocalPort() );
	d1.IkeGroupCombobox->setCurrentText( currentProfile->getIkeGroup() );
	d1.PerfectForwardSecrecyCombobox->setCurrentText( currentProfile->getPerfectForwardSecrety() );
	d1.NtDomainNameLineedit->setText( currentProfile->getNtDomainName() );
	d1.ApplicationVersionLineedit->setText( currentProfile->getApplicationVersion() );

	if ( currentProfile->getUseAdvancedSettings() ) {
		d1.LocalPortCheckbox->setEnabled ( true );
		d1.IkeGroupCheckbox->setEnabled ( true );
		d1.PerfectForwardSecurityCheckbox->setEnabled ( true );
		d1.SingleDesCheckbox->setEnabled ( true );
		d1.NtDomainNameCheckbox->setEnabled ( true );
		d1.ApplicationVersionCheckbox->setEnabled ( true );
		d1.UseGlobalIpsecSecretCheckbox->setEnabled ( true );
		d1.AdvancedSettingsCheckbox->setChecked ( true );

		if ( currentProfile->getUseSingleDes() ) {
			d1.SingleDesCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseIkeGroup() ) {
			d1.IkeGroupCheckbox->setChecked ( true );
		}

		if ( currentProfile->getUseGlobalIpsecSecret() ) {
			d1.UseGlobalIpsecSecretCheckbox->setChecked ( true );
		}

		if ( currentProfile->getUsePerfectForwardSecrety() ) {
			d1.PerfectForwardSecrecyCombobox->setEnabled ( true );
			d1.PerfectForwardSecurityCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseNtDomainName() ) {
			d1.NtDomainNameLineedit->setEnabled ( true );
			d1.NtDomainNameCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseApplicationVersion() ) {
			d1.ApplicationVersionLineedit->setEnabled ( true );
			d1.ApplicationVersionCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseLocalPort() ) {
			d1.LocalPortSpinbox->setEnabled ( true );
			d1.LocalPortCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseIkeGroup() ) {
			d1.IkeGroupCombobox->setEnabled ( true );
		}
	}

	d1.exec();

	if ( d1.AdvancedSettingsCheckbox->isChecked() ) {
		currentProfile->setUseAdvancedSettings( true );
	} else {
		currentProfile->setUseAdvancedSettings( false );
	}
	if ( d1.NtDomainNameCheckbox->isChecked() ) {
		currentProfile->setNtDomainName( d1.NtDomainNameLineedit->text() );
		currentProfile->setUseNtDomainName( true );
	} else {
		currentProfile->setUseNtDomainName( false );
	}

	if ( d1.ApplicationVersionCheckbox->isChecked() ) {
		currentProfile->setApplicationVersion( d1.ApplicationVersionLineedit->text() );
		currentProfile->setUseApplicationVersion( true );
	} else {
		currentProfile->setUseApplicationVersion( false );
	}

	if ( d1.IkeGroupCheckbox->isChecked() ) {
		currentProfile->setIkeGroup( d1.IkeGroupCombobox->currentText() );
		currentProfile->setUseIkeGroup( true );
	} else {
		currentProfile->setUseIkeGroup( false );
	}

	if ( d1.PerfectForwardSecurityCheckbox->isChecked() ) {
		currentProfile->setPerfectForwardSecrety( d1.PerfectForwardSecrecyCombobox->currentText() );
		currentProfile->setUsePerfectForwardSecrety( true );
	} else {
		currentProfile->setUsePerfectForwardSecrety( false );
	}


	if ( d1.SingleDesCheckbox->isChecked() ) {
		currentProfile->setUseSingleDes( true );
	} else {
		currentProfile->setUseSingleDes( false );
	}

	if ( d1.UseGlobalIpsecSecretCheckbox->isChecked() ) {
		currentProfile->setUseGlobalIpsecSecret( true );
	} else {
		currentProfile->setUseGlobalIpsecSecret( false );
	}


	if ( d1.LocalPortCheckbox->isChecked() ) {
		currentProfile->setLocalPort( d1.LocalPortSpinbox->value() );
		currentProfile->setUseLocalPort( true );
	} else {
		currentProfile->setUseLocalPort( false );
	}

	saveSessionClicked();

}

void ProfileWidget::pingIpToggled( bool ) {
	if ( PingCheckBox->isChecked() ) {
		PingIPLineEdit->setEnabled( true );
	} else {
		PingIPLineEdit->setEnabled( false );
	}
}

void ProfileWidget::useSpecialRemoteIDClicked() {
	if ( UseSpecialRemoteIDcheckBox->isChecked() )
		RemoteIDLineEdit->setEnabled( true );
	else
		RemoteIDLineEdit->setEnabled( false );
}






#include "profilewidget.moc"
